package com.espectaclia.entities;

import java.text.DecimalFormat;
import java.util.List;

import javax.persistence.Entity;
import javax.persistence.JoinColumn;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.OneToMany;
import javax.persistence.PrimaryKeyJoinColumn;

import org.hibernate.annotations.LazyCollection;
import org.hibernate.annotations.LazyCollectionOption;

import com.espectaclia.enumerations.EstatFuncio;
import com.espectaclia.enumerations.EstatReserva;

@Entity(name = "Administrador")
@PrimaryKeyJoinColumn(name="id")
@NamedQueries({
@NamedQuery(name="Administrador.findAll",query="SELECT a FROM Administrador a")})
public class Administrador extends Usuari {
	
	@OneToMany
	@JoinColumn(name = "idAdministrador")
	@LazyCollection(LazyCollectionOption.FALSE)
	private List<Taquiller> taquillers;
	
	@OneToMany
	@JoinColumn(name = "idAdministrador")
	@LazyCollection(LazyCollectionOption.FALSE)
	private List<Recinte> recintes;

	public List<Taquiller> getTaquillers() {
		return taquillers;
	}

	public void setTaquillers(List<Taquiller> taquillers) {
		this.taquillers = taquillers;
	}
	
	public List<Recinte> getRecintes() {
		return recintes;
	}

	public void setRecintes(List<Recinte> recintes) {
		this.recintes = recintes;
	}
	
	public int getNumTotalEspectacles() {
		int numEspectacles = 0;
		
		for (Recinte recinte : this.recintes) {
			numEspectacles += recinte.getEspectacles().size();
		}
		
		return numEspectacles;
	}
	
	public int getNumTotalFuncionsActives() {
		int numFuncions = 0;
		
		for (Recinte recinte : this.recintes) {
			for (Espectacle espectacle : recinte.getEspectacles()) {
				for (Funcio funcio : espectacle.getFuncions()) {
					if (funcio.getEstat() == EstatFuncio.ACTIVA) ++numFuncions;
				}
			}
		}
		
		return numFuncions;
	}
	
	public int getNumTotalReservesConfirmades() {
		int numReserves = 0;
		
		for (Recinte recinte : this.recintes) {
			for (Espectacle espectacle : recinte.getEspectacles()) {
				for (Funcio funcio : espectacle.getFuncions()) {
					if (funcio.getEstat() == EstatFuncio.ACTIVA) {
						for (Reserva reserva : funcio.getReserves()) {
							if (reserva.getEstat() == EstatReserva.CONFIRMADA) ++numReserves;
						}
					}
				}
			}
		}
		
		return numReserves;
	}
	
	public int getNumTotalReservesVenudes() {
		int numReserves = 0;
		
		for (Recinte recinte : this.recintes) {
			for (Espectacle espectacle : recinte.getEspectacles()) {
				for (Funcio funcio : espectacle.getFuncions()) {
					if (funcio.getEstat() == EstatFuncio.ACTIVA) {
						for (Reserva reserva : funcio.getReserves()) {
							if (reserva.getEstat() == EstatReserva.VENUDA) ++numReserves;
						}
					}
				}
			}
		}
		
		return numReserves;
	}
	
	public String getRecaptacioTotal() {
		double recaptacio = 0;
		
		for (Recinte recinte : this.recintes) {
			for (Espectacle espectacle : recinte.getEspectacles()) {
				for (Funcio funcio : espectacle.getFuncions()) {
					if (funcio.getEstat() == EstatFuncio.ACTIVA) {
						for (Reserva reserva : funcio.getReserves()) {
							if (reserva.getEstat() == EstatReserva.VENUDA) recaptacio += reserva.getPreuFinal();
						}
					}
				}
			}
		}
		
		DecimalFormat format = new DecimalFormat("0.## €");
		return String.valueOf(format.format(recaptacio));
	}
	
}