package com.espectaclia.entities;

import java.util.ArrayList;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.OneToMany;

import org.hibernate.annotations.LazyCollection;
import org.hibernate.annotations.LazyCollectionOption;

import com.espectaclia.enumerations.EstatFuncio;
import com.espectaclia.enumerations.TipusEspectacle;

@Entity(name = "Espectacle")
@NamedQueries({
@NamedQuery(name="Espectacle.findAll",query="SELECT e FROM Espectacle e")})
public class Espectacle {
	
	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
    @Column(name="id",nullable=false)
    private int id;
	
	@Column(name="nom",nullable=false,length=30)
    private String nom;
    
    @Column(name="descripcio",nullable=false,length=2000)
    private String descripcio;
	
    @Enumerated(EnumType.ORDINAL)
    @Column(name="tipus",nullable=false)
    private TipusEspectacle tipus;
    
    @Column(name="genere",nullable=true,length=60)
    private String genere;
    
    @Column(name="autor",nullable=true,length=80)
    private String autor;
    
    @Column(name="interprets",nullable=true,length=255)
    private String interprets;
    
    @Column(name="duracio",nullable=false,length=255)
    private Long duracio;
    
    @OneToMany(cascade=CascadeType.ALL,mappedBy="espectacle",orphanRemoval=true)
	@LazyCollection(LazyCollectionOption.FALSE)
	private List<Funcio> funcions;
    
    @OneToMany(cascade=CascadeType.ALL,mappedBy="espectacle",orphanRemoval=true)
	@LazyCollection(LazyCollectionOption.FALSE)
	private List<Descompte> descomptes;
    
    @OneToMany(cascade=CascadeType.ALL,mappedBy="espectacle",orphanRemoval=true)
   	@LazyCollection(LazyCollectionOption.FALSE)
   	private List<PreuLocalitatZona> preuLocalitatZones;
    
    @OneToMany(cascade=CascadeType.ALL,mappedBy="espectacle",orphanRemoval=true)
   	@LazyCollection(LazyCollectionOption.FALSE)
   	private List<Valoracio> valoracions;
    
    @ManyToOne
    @JoinColumn(name="idRecinte",nullable=false)
    @LazyCollection(LazyCollectionOption.FALSE) 
 	private Recinte recinte;

	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public String getNom() {
		return nom;
	}

	public void setNom(String nom) {
		this.nom = nom;
	}

	public String getDescripcio() {
		return descripcio;
	}

	public void setDescripcio(String descripcio) {
		this.descripcio = descripcio;
	}

	public TipusEspectacle getTipus() {
		return tipus;
	}

	public void setTipus(TipusEspectacle tipus) {
		this.tipus = tipus;
	}

	public String getGenere() {
		return genere;
	}

	public void setGenere(String genere) {
		this.genere = genere;
	}

	public String getAutor() {
		return autor;
	}

	public void setAutor(String autor) {
		this.autor = autor;
	}

	public String getInterprets() {
		return interprets;
	}

	public void setInterprets(String interprets) {
		this.interprets = interprets;
	}

	public Long getDuracio() {
		return duracio;
	}

	public void setDuracio(Long duracio) {
		this.duracio = duracio;
	}

	public List<Funcio> getFuncions() {
		return funcions;
	}

	public void setFuncions(List<Funcio> funcions) {
		this.funcions = funcions;
	}
	
	public List<Descompte> getDescomptes() {
		return descomptes;
	}

	public void setDescomptes(List<Descompte> descomptes) {
		this.descomptes = descomptes;
	}
	
	public List<PreuLocalitatZona> getPreuLocalitatZones() {
		return preuLocalitatZones;
	}

	public void setPreuLocalitatZones(List<PreuLocalitatZona> preuLocalitatZones) {
		this.preuLocalitatZones = preuLocalitatZones;
	}
	
	public List<Valoracio> getValoracions() {
		return valoracions;
	}

	public void setValoracions(List<Valoracio> valoracions) {
		this.valoracions = valoracions;
	}

	public Recinte getRecinte() {
		return recinte;
	}

	public void setRecinte(Recinte recinte) {
		this.recinte = recinte;
	}
	
	public String getDescripcioTipus() {
		switch(tipus) {
		case OBRA_TEATRAL:
			return "Obra teatral";
		case DANSA:
			return "Dansa";
		case PELLICULA:
			return "Pel·lícula";
		case EXPOSICIO:
			return "Exposició";
		case CONCERT:
			return "Concert";
		case CONFERENCIA:
			return "Conferència";
		case ESDEVENIMENT_ESPORTIU:
			return "Esdeveniment esportiu";
		case CONGRES:
			return "Congrés";
		case ALTRES:
			return "Altres";
		default:
			return "";
		}
	}
	
	public boolean teFuncionsActives() {
		if (funcions.size() > 0) {
			for (Funcio funcio : funcions) {
				if (funcio.getEstat() == EstatFuncio.ACTIVA) return true;
			}
		}
		return false;
	}
	
	public int getValoracioMitja() {
		int valMitja = 0;
		
		if (this.valoracions != null && this.valoracions.size() > 0) {
			for (Valoracio valoracio : this.valoracions) {
				valMitja += valoracio.getPuntuacio().ordinal();
			}
			valMitja = Math.round(valMitja / this.valoracions.size()) + 1;
		}
		
		return valMitja;
	}
	
	public String getValoracioMitjaText() {
		int valMitja = getValoracioMitja();
		String valMitjaText = "";
		
		if (valMitja == 0) {
			return "Sense valoracions";
		} else {
			switch (valMitja) {
			case 1:
				valMitjaText = "Malament";
				break;
			case 2:
				valMitjaText = "Regular";
				break;
			case 3:
				valMitjaText = "Bé";
				break;
			case 4:
				valMitjaText = "Molt bé";
				break;
			case 5:
				valMitjaText = "Excel·lent";
				break;
			}
			
			valMitjaText += " ("+this.valoracions.size()+" "+(this.valoracions.size() > 1 ? "valoracions" : "valoració")+")";
		}
		
		return valMitjaText;
	}
	
	public boolean teValoracionsAmbComentaris() {
		boolean ambComentaris = false;
		
		if (this.valoracions != null) {
			for (Valoracio valoracio : this.valoracions) {
				if (valoracio.getComentari() != null && valoracio.getComentari().trim().length() > 0) {
					ambComentaris = true;
					break;
				}
			}
		}
		
		return ambComentaris;
	}
	
	public List<Valoracio> getValoracionsAmbComentaris() {
		List<Valoracio> comentaris = new ArrayList<Valoracio>();
		
		if (this.valoracions != null) {
			for (Valoracio valoracio : this.valoracions) {
				if (valoracio.getComentari() != null && valoracio.getComentari().trim().length() > 0) {
					comentaris.add(valoracio);
				}
			}
		}
		
		return comentaris;
	}
 	
}