package com.espectaclia.entities;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.OneToMany;

import org.hibernate.annotations.LazyCollection;
import org.hibernate.annotations.LazyCollectionOption;

import com.espectaclia.enumerations.EstatFuncio;
import com.espectaclia.enumerations.EstatReserva;

@Entity(name = "Funcio")
@NamedQueries({
@NamedQuery(name="Funcio.findAll",query="SELECT f FROM Funcio f")})
public class Funcio {
	
	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
    @Column(name="id",nullable=false)
    private int id;
	
	@Column(name="data",nullable=false)
    private Date data;
	
	@Enumerated(EnumType.ORDINAL)
    @Column(name="estat",nullable=false)
    private EstatFuncio estat;
	
	@OneToMany(cascade=CascadeType.ALL,mappedBy="funcio",orphanRemoval=true)
   	@LazyCollection(LazyCollectionOption.FALSE)
   	private List<Reserva> reserves;
	
	@ManyToOne 
    @JoinColumn(name="idEspectacle",nullable=false)
    @LazyCollection(LazyCollectionOption.FALSE) 
 	private Espectacle espectacle;

	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public Date getData() {
		return data;
	}

	public void setData(Date data) {
		this.data = data;
	}

	public EstatFuncio getEstat() {
		return estat;
	}

	public void setEstat(EstatFuncio estat) {
		this.estat = estat;
	}

	public List<Reserva> getReserves() {
		return reserves;
	}

	public void setReserves(List<Reserva> reserves) {
		this.reserves = reserves;
	}

	public Espectacle getEspectacle() {
		return espectacle;
	}

	public void setEspectacle(Espectacle espectacle) {
		this.espectacle = espectacle;
	}
	
	public String getFormattedData() {
		return new SimpleDateFormat("dd-MM-yyyy HH:mm").format(data);
	}
	
	public String getDescripcioEstat() {
		switch(estat) {
		case ACTIVA:
			return "Activa";
		case CANCELLADA:
			return "Cancel·lada";
		default:
			return "";
		}
	}
	
	public boolean teReservesPendents() {
		if (reserves.size() > 0) {
			for (Reserva reserva : reserves) {
				if (reserva.getEstat() == EstatReserva.CONFIRMADA) return true;
			}
		}
		return false;
	}
	
	public boolean teReservesVenudes() {
		if (reserves.size() > 0) {
			for (Reserva reserva : reserves) {
				if (reserva.getEstat() == EstatReserva.VENUDA) return true;
			}
		}
		return false;
	}
	
	public Integer getNumReservesClients() {
		Integer numReservesClients = 0;
		
		if (reserves.size() > 0) {
			for (Reserva reserva : reserves) {
				if (reserva.getEstat() == EstatReserva.CONFIRMADA || reserva.getEstat() == EstatReserva.TEMPORAL || reserva.getEstat() == EstatReserva.VENUDA) ++numReservesClients;
			}
		}
		return numReservesClients;
	}
	
}