package com.espectaclia.entities;

import java.util.ArrayList;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.OneToMany;

import org.apache.commons.lang3.builder.EqualsBuilder;
import org.apache.commons.lang3.builder.HashCodeBuilder;
import org.hibernate.annotations.LazyCollection;
import org.hibernate.annotations.LazyCollectionOption;

import com.espectaclia.enumerations.TipusRecinte;

@Entity(name = "Recinte")
@NamedQueries({
@NamedQuery(name="Recinte.findAll",query="SELECT r FROM Recinte r")})
public class Recinte {
	
	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
    @Column(name="id",nullable=false)
    private Integer id;
    
	@Column(name="nom",nullable=false,length=30)
    private String nom;
    
    @Column(name="descripcio",nullable=true,length=255)
    private String descripcio;
	
    @Enumerated(EnumType.ORDINAL)
    @Column(name="tipus",nullable=false)
    private TipusRecinte tipus;

	@Column(name="aforament",nullable=false)
    private Long aforament;
    
    @Column(name="adreca",nullable=true,length=80)
    private String adreca;
    
    @Column(name="emailContacte",nullable=true,length=40)
    private String emailContacte;
    
    @Column(name="telefonContacte",nullable=true)
    private Integer telefonContacte;
    
    @OneToMany
	@JoinColumn(name = "idRecinte")
	@LazyCollection(LazyCollectionOption.FALSE)
	private List<Taquiller> taquillers;
    
    @OneToMany(cascade=CascadeType.ALL,mappedBy="recinte",orphanRemoval=true)
	@LazyCollection(LazyCollectionOption.FALSE)
	private List<Zona> zones;
    
    @OneToMany(cascade=CascadeType.ALL,mappedBy="recinte",orphanRemoval=true)
	@LazyCollection(LazyCollectionOption.FALSE)
	private List<Espectacle> espectacles = new ArrayList<Espectacle>();

	@ManyToOne 
    @JoinColumn(name="idAdministrador",nullable=false)
    @LazyCollection(LazyCollectionOption.FALSE) 
 	private Administrador administrador;
    
    public Integer getId() {
		return id;
	}
    
    public void setId(Integer id) {
		this.id = id;
	}

    public String getNom() {
		return nom;
	}

	public void setNom(String nom) {
		this.nom = nom;
	}
	
	public String getDescripcio() {
		return descripcio;
	}

	public void setDescripcio(String descripcio) {
		this.descripcio = descripcio;
	}
	
	public TipusRecinte getTipus() {
		return tipus;
	}

	public void setTipus(TipusRecinte tipus) {
		this.tipus = tipus;
	}

	public Long getAforament() {
		return aforament;
	}

	public void setAforament(Long aforament) {
		this.aforament = aforament;
	}
	
	public String getAdreca() {
		return adreca;
	}

	public void setAdreca(String adreca) {
		this.adreca = adreca;
	}

	public String getEmailContacte() {
		return emailContacte;
	}

	public void setEmailContacte(String emailContacte) {
		this.emailContacte = emailContacte;
	}

	public Integer getTelefonContacte() {
		return telefonContacte;
	}

	public void setTelefonContacte(Integer telefonContacte) {
		this.telefonContacte = telefonContacte;
	}

	public List<Taquiller> getTaquillers() {
		return taquillers;
	}

	public void setTaquillers(List<Taquiller> taquillers) {
		this.taquillers = taquillers;
	}
	
	public List<Zona> getZones() {
		return zones;
	}

	public void setZones(List<Zona> zones) {
		this.zones = zones;
	}

	public List<Espectacle> getEspectacles() {
		return espectacles;
	}

	public void setEspectacles(List<Espectacle> espectacles) {
		this.espectacles = espectacles;
	}
	
	public Administrador getAdministrador() {
		return administrador;
	}

	public void setAdministrador(Administrador administrador) {
		this.administrador = administrador;
	}
	
	public String getDescripcioTipus() {
		switch(tipus) {
		case TEATRE:
			return "Teatre";
		case CINEMA:
			return "Cinema";
		case MUSEU:
			return "Museu";
		case SALA_CONCERTS:
			return "Sala de concerts";
		case SALA_EXPOSICIONS:
			return "Sala d'exposicions";
		case SALA_POLIVALENT:
			return "Sala polivalent";
		case SALO_CONGRESSOS:
			return "Saló de congressos";
		case CENTRE_CULTURAL:
			return "Centre cultural";
		case POLIESPORTIU:
			return "Poliesportiu";
		case ESTADI:
			return "Estadi";
		case FIRA:
			return "Fira";
		case ALTRES:
			return "Altres";
		default:
			return "";
		}
	}
	
	@Override
    public int hashCode() {
        return new HashCodeBuilder(17, 31).
            append(id).
            append(nom).
            append(tipus).
            toHashCode();
    }
	
	@Override
	public boolean equals(Object obj) {
		if (obj == this) {
			return true;
		}
		if (obj == null || obj.getClass() != this.getClass()) {
			return false;
		}
		
		Recinte rec = (Recinte) obj;
		return new EqualsBuilder().
            append(id, rec.id).
            append(nom, rec.nom).
            append(tipus, rec.tipus).
            isEquals();
//		return id == rec.id
//			&& (nom == rec.nom 
//				|| (nom != null && nom.equals(rec.getNom())))
//			&& (tipus == rec.tipus 
//				|| (tipus != null && tipus.equals(rec.getTipus())));
	}
    
}