package com.espectaclia.entities;

import java.util.Date;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.OneToMany;

import org.apache.commons.lang3.builder.EqualsBuilder;
import org.apache.commons.lang3.builder.HashCodeBuilder;
import org.hibernate.annotations.LazyCollection;
import org.hibernate.annotations.LazyCollectionOption;

import com.espectaclia.enumerations.EstatReserva;

@Entity(name = "Reserva")
@NamedQueries({
@NamedQuery(name="Reserva.findAll",query="SELECT r FROM Reserva r")})
public class Reserva {
	
	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
    @Column(name="id",nullable=false)
    private Integer id;
	
	@Enumerated(EnumType.ORDINAL)
    @Column(name="estat",nullable=false)
    private EstatReserva estat;
	
	@Column(name="preuFinal",nullable=false)
    private double preuFinal;
	
	@Column(name="numLocalitats",nullable=false)
    private int numLocalitats;
	
	@Column(name="dataCreacio",nullable=false)
    private Date dataCreacio;
    
    @Column(name="dataUltModificacio",nullable=true)
    private Date dataUltModificacio;
    
    @Column(name="codiQR",nullable=false)
    private String codiQR;

    @ManyToOne
	@JoinColumn(name="idClient",nullable=false)
	private Client client;
	
    @ManyToOne
	@JoinColumn(name="idFuncio",nullable=false)
	private Funcio funcio;
	
	@ManyToOne
	@JoinColumn(name="idZona",nullable=false)
	private Zona zona;
    
	@OneToMany(cascade=CascadeType.ALL,mappedBy="reserva",orphanRemoval=true)
   	@LazyCollection(LazyCollectionOption.FALSE)
   	private List<Localitat> localitats;
    
	public Integer getId() {
		return id;
	}

	public void setId(Integer id) {
		this.id = id;
	}
	
	public EstatReserva getEstat() {
		return estat;
	}

	public void setEstat(EstatReserva estat) {
		this.estat = estat;
	}

	public double getPreuFinal() {
		return preuFinal;
	}

	public void setPreuFinal(double preuFinal) {
		this.preuFinal = preuFinal;
	}
	
	public int getNumLocalitats() {
		return numLocalitats;
	}

	public void setNumLocalitats(int numLocalitats) {
		this.numLocalitats = numLocalitats;
	}

	public Date getDataCreacio() {
		return dataCreacio;
	}

	public void setDataCreacio(Date dataCreacio) {
		this.dataCreacio = dataCreacio;
	}

	public Date getDataUltModificacio() {
		return dataUltModificacio;
	}

	public void setDataUltModificacio(Date dataUltModificacio) {
		this.dataUltModificacio = dataUltModificacio;
	}
	
	public String getCodiQR() {
		return codiQR;
	}

	public void setCodiQR(String codiQR) {
		this.codiQR = codiQR;
	}
	
	public Client getClient() {
		return client;
	}

	public void setClient(Client client) {
		this.client = client;
	}

	public Funcio getFuncio() {
		return funcio;
	}

	public void setFuncio(Funcio funcio) {
		this.funcio = funcio;
	}
	
	public Zona getZona() {
		return zona;
	}

	public void setZona(Zona zona) {
		this.zona = zona;
	}

	public List<Localitat> getLocalitats() {
		return localitats;
	}

	public void setLocalitats(List<Localitat> localitats) {
		this.localitats = localitats;
	}
	
	public String getDescripcioEstat() {
		if (estat != null) {
			switch(estat) {
			case BLOQUEJADA:
				return "Bloquejada";
			case TEMPORAL:
				return "Temporal";
			case VENUDA:
				return "Venuda";
			case CANCELLADA:
				return "Cancel·lada";
			case CONFIRMADA:
				return "Confirmada";
			default:
				return "";
			}
		} else {
			return "";
		}
	}
	
	@Override
    public int hashCode() {
        return new HashCodeBuilder(19, 23).
            append(id).
            append(client.getId()).
            append(funcio.getId()).
            append(zona.getId()).
            append(estat).
            toHashCode();
    }
	
	@Override
	public boolean equals(Object obj) {
		if (obj == this) {
			return true;
		}
		if (obj == null || obj.getClass() != this.getClass()) {
			return false;
		}
		
		Reserva res = (Reserva) obj;
		return new EqualsBuilder().
            append(id, res.id).
            append(client.getId(), res.client.getId()).
            append(funcio.getId(), res.funcio.getId()).
            append(zona.getId(), res.zona.getId()).
            append(estat, res.estat).
            isEquals();
	}
	
}