package com.espectaclia.entities;

import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.OneToMany;
import javax.persistence.Transient;

import org.hibernate.annotations.LazyCollection;
import org.hibernate.annotations.LazyCollectionOption;

import com.espectaclia.enumerations.EstatReserva;

@Entity(name = "Zona")
@NamedQueries({
@NamedQuery(name="Zona.findAll",query="SELECT z FROM Zona z")})
public class Zona {
	
	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
    @Column(name="id",nullable=false)
    private int id;
	
	@Column(name="nom",nullable=false,length=15)
	private String nom;
	
	@Column(name="aforament",nullable=false)
	private Long aforament;
	
	@Column(name="coordInici",nullable=false,length=3)
	private String coordInici;
	
	@Column(name="coordFi",nullable=false,length=3)
	private String coordFi;
	
	@Column(name="numFiles",nullable=true)
	private Long numFiles;
	
	@Column(name="numSeientsFila",nullable=true)
	private Long numSeientsFila;
	
	@OneToMany(cascade=CascadeType.ALL,mappedBy="zona",orphanRemoval=true)
   	@LazyCollection(LazyCollectionOption.FALSE)
   	private List<Localitat> localitats;
	
	@OneToMany(cascade=CascadeType.ALL,mappedBy="zona",orphanRemoval=true)
   	@LazyCollection(LazyCollectionOption.FALSE)
   	private List<PreuLocalitatZona> preuLocalitatZones;
	
	@OneToMany(cascade=CascadeType.ALL,mappedBy="zona",orphanRemoval=true)
   	@LazyCollection(LazyCollectionOption.FALSE)
   	private List<Reserva> reserves;

	@ManyToOne 
    @JoinColumn(name="idRecinte",nullable=false)
    @LazyCollection(LazyCollectionOption.FALSE) 
 	private Recinte recinte;
	
	@Transient
	private List<Localitat> localitatsPatiButaques;

	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public String getNom() {
		return nom;
	}

	public void setNom(String nom) {
		this.nom = nom;
	}

	public Long getAforament() {
		return aforament;
	}

	public void setAforament(Long aforament) {
		this.aforament = aforament;
	}

	public String getCoordInici() {
		return coordInici;
	}

	public void setCoordInici(String coordInici) {
		this.coordInici = coordInici;
	}

	public String getCoordFi() {
		return coordFi;
	}

	public void setCoordFi(String coordFi) {
		this.coordFi = coordFi;
	}

	public Long getNumFiles() {
		return numFiles;
	}

	public void setNumFiles(Long numFiles) {
		this.numFiles = numFiles;
	}

	public Long getNumSeientsFila() {
		return numSeientsFila;
	}

	public void setNumSeientsFila(Long numSeientsFila) {
		this.numSeientsFila = numSeientsFila;
	}
	
	public List<Localitat> getLocalitats() {
		return localitats;
	}

	public void setLocalitats(List<Localitat> localitats) {
		this.localitats = localitats;
	}
	
	public List<PreuLocalitatZona> getPreuLocalitatZones() {
		return preuLocalitatZones;
	}

	public void setPreuLocalitatZones(List<PreuLocalitatZona> preuLocalitatZones) {
		this.preuLocalitatZones = preuLocalitatZones;
	}
	
	public List<Reserva> getReserves() {
		return reserves;
	}

	public void setReserves(List<Reserva> reserves) {
		this.reserves = reserves;
	}

	public Recinte getRecinte() {
		return recinte;
	}

	public void setRecinte(Recinte recinte) {
		this.recinte = recinte;
	}
	
	public List<Localitat> getLocalitatsPatiButaques() {
		return localitatsPatiButaques;
	}

	public void setLocalitatsPatiButaques(List<Localitat> localitatsPatiButaques) {
		this.localitatsPatiButaques = localitatsPatiButaques;
	}
	
	public boolean teReservesPendents(Integer idEspectacle) {
		if (reserves.size() > 0) {
			for (Reserva reserva : reserves) {
				if (idEspectacle != null) { // Per a un espectacle concret
					if (reserva.getFuncio().getEspectacle().getId() == idEspectacle && reserva.getEstat() == EstatReserva.CONFIRMADA) return true;
				} else { // Per a qualsevol espectacle
					if (reserva.getEstat() == EstatReserva.CONFIRMADA) return true;
				}
			}
		}
		return false;
	}
	
}