package com.espectaclia.managed_beans;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.ViewScoped;
import javax.faces.component.UIInput;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.servlet.ServletContext;

import com.espectaclia.daos.ClientDAO;
import com.espectaclia.daos.UsuariDAO;
import com.espectaclia.entities.Client;
import com.espectaclia.entities.Usuari;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.context.support.WebApplicationContextUtils;

@ManagedBean(name="clientMB")
@ViewScoped
public class ClientMB implements Serializable{

	private static final long serialVersionUID = 1L;
	private static final String messageCloseButton = "<button type=\"button\" class=\"close\" onclick=\"$(this).parent().fadeOut()\"><span aria-hidden=\"true\">×</span></button>";
	
	@ManagedProperty(value="#{loginMB}")
    private LoginMB loginMB;
	
	@Autowired
	private UsuariDAO usuariDAO;
	
	@Autowired
	private ClientDAO clientDAO;
	
	private Client client;
	private Date currentDate = new Date();
	
	@PostConstruct
	public void init() {
		// Importem l'Autowired (Spring) al ManagedBean (JSF) per poder injectar els DAOs
		ExternalContext externalContext = FacesContext.getCurrentInstance().getExternalContext();
        ServletContext servletContext = (ServletContext) externalContext.getContext();
        WebApplicationContextUtils.getRequiredWebApplicationContext(servletContext).getAutowireCapableBeanFactory().autowireBean(this);
		
    	this.client = new Client();
    }
	
	public String crear() {
		try {
			// Comprovem si ja existeix un altre usuari amb el username indicat
			List<Usuari> usuaris = usuariDAO.cercarPerPropietat("username", client.getUsername());
			if (usuaris.size() > 0) {
				UIInput uiInputUsuari = (UIInput) FacesContext.getCurrentInstance().getViewRoot().findComponent("client_form:client_usuari_input");
				uiInputUsuari.setValid(false);
				
				FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "L'usuari \""+client.getUsername()+"\" ja existeix. Tria un altre usuari." + messageCloseButton, "");
	            FacesContext.getCurrentInstance().addMessage(null, message);
	            return null;
			}
			
			// Comprovem si ja existeix un altre client amb l'e-mail indicat
			List<Client> clients = clientDAO.cercarPerPropietat("email", client.getEmail());
			if (clients.size() > 0) {
				UIInput uiInputUsuari = (UIInput) FacesContext.getCurrentInstance().getViewRoot().findComponent("client_form:client_email_input");
				uiInputUsuari.setValid(false);
				
				FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "Ja existeix un client amb l'e-mail indicat. Tria un altre e-mail." + messageCloseButton, "");
	            FacesContext.getCurrentInstance().addMessage(null, message);
	            return null;
			}
			
			client.setDataAlta(new Date());
			
			clientDAO.persistir(this.client);

			// El nou client s'ha creat amb èxit
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_INFO, "El teu client s'ha creat amb èxit. Ja pots identificar-te." + messageCloseButton, "");
			FacesContext.getCurrentInstance().addMessage(null, message);
			FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);
            return "/views/public/login.xhtml?faces-redirect=true";
		} catch (Exception e) {
			// S'ha produït un error desconegut intentant crear el nou client
			e.printStackTrace();
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "No s'ha pogut crear el teu client. Si us plau, comprova que les dades introduïdes siguin correctes." + messageCloseButton, "");
            FacesContext.getCurrentInstance().addMessage(null, message);
            return null;
		}
	}
	
	public String guardar() {
		try {
			// Comprovem si ja existeix un altre usuari amb el username indicat (que no sigui el propi client)
			List<Usuari> usuaris = usuariDAO.cercarPerPropietat("username", client.getUsername());
			if (usuaris.size() > 0 && usuaris.get(0).getId() != client.getId()) {
				UIInput uiInputUsuari = (UIInput) FacesContext.getCurrentInstance().getViewRoot().findComponent("client_form:client_usuari_input");
				uiInputUsuari.setValid(false);
				
				FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "L'usuari \""+client.getUsername()+"\" ja existeix. Tria un altre usuari." + messageCloseButton, "");
	            FacesContext.getCurrentInstance().addMessage(null, message);
	            return null;
			}
			
			// Comprovem si ja existeix un altre client amb l'e-mail indicat (que no sigui el propi client)
			List<Client> clients = clientDAO.cercarPerPropietat("email", client.getEmail());
			if (clients.size() > 0 && usuaris.get(0).getId() != client.getId()) {
				UIInput uiInputUsuari = (UIInput) FacesContext.getCurrentInstance().getViewRoot().findComponent("client_form:client_email_input");
				uiInputUsuari.setValid(false);
				
				FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "Ja existeix un client amb l'e-mail indicat. Tria un altre e-mail." + messageCloseButton, "");
	            FacesContext.getCurrentInstance().addMessage(null, message);
	            return null;
			}
			
			client.setDataUltModificacio(new Date());
			
			clientDAO.unir(this.client);
			
			loginMB.setLoggedUser(this.client);

			// El client s'ha modificat amb èxit
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_INFO, "El teu perfil s'ha modificat amb èxit." + messageCloseButton, "");
			FacesContext.getCurrentInstance().addMessage(null, message);
			FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);
			return "/views/app/client/modificar_client.xhtml?faces-redirect=true";
		} catch (Exception e) {
			// S'ha produït un error desconegut intentant modificar el client
			e.printStackTrace();
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "No s'ha pogut modificar el teu perfil. Si us plau, comprova que les dades modificades siguin correctes." + messageCloseButton, "");
            FacesContext.getCurrentInstance().addMessage(null, message);
            return null;
		}
	}
	
	public String eliminar() {
		try {
			Client loggedUser = (Client) loginMB.getLoggedUser();
			
			// Comprovem si el client té cap reserva pendent (Estat: CONFIRMADA)
			if (!loggedUser.teReservesPendents()) {
				clientDAO.eliminar(loggedUser.getId());
				
				// El client s'ha eliminat amb èxit. Fem logout.
				FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_INFO, "El teu usuari s'ha eliminat amb èxit." + messageCloseButton, "");
				FacesContext.getCurrentInstance().addMessage(null, message);
				FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);

				return loginMB.logout();
			} else {
				FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "No pots eliminar el teu usuari. Tens reserves pendents." + messageCloseButton, "");
	            FacesContext.getCurrentInstance().addMessage(null, message);
			}
		} catch (Exception e) {
			// S'ha produït un error desconegut intentant eliminar el client
			e.printStackTrace();
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "S'ha produït un error i no s'ha pogut eliminar el teu usuari." + messageCloseButton, "");
            FacesContext.getCurrentInstance().addMessage(null, message);
		}
		return null;
	}
	
	public void iniciarModificacioClient() {
		if (loginMB.getTipusUser().equalsIgnoreCase("CLIENT")) {
			Client loggedUser = (Client) loginMB.getLoggedUser();
			
			client.setId(loggedUser.getId());
			client.setNom(loggedUser.getNom());
			client.setCognoms(loggedUser.getCognoms());
			client.setNif(loggedUser.getNif());
			client.setDataNaixement(loggedUser.getDataNaixement());
			client.setAdreca(loggedUser.getAdreca());
			client.setEmail(loggedUser.getEmail());
			client.setTelefon(loggedUser.getTelefon());
			client.setUsername(loggedUser.getUsername());
			client.setPassword(loggedUser.getPassword());
			client.setDataAlta(loggedUser.getDataAlta());
			client.setReserves(loggedUser.getReserves());
			client.setValoracions(loggedUser.getValoracions());
		}
	}
	
	public String getMessageCloseButton() {
		return messageCloseButton;
	}
	
	public LoginMB getLoginMB() {
		return loginMB;
	}
	
	public void setLoginMB(LoginMB loginMB) {
		this.loginMB = loginMB;
	}
	
	public Client getClient() {
		return client;
	}
	
	public void setClient(Client client) {
		this.client = client;
	}
	
	public Date getCurrentDate() {
        return this.currentDate;
    }
	
	public void setCurrentDate(Date currentDate) {
		this.currentDate = currentDate;
	}
	
}