package com.espectaclia.managed_beans;

import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.faces.application.ConfigurableNavigationHandler;
import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ViewScoped;
import javax.faces.component.UIInput;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.faces.model.SelectItem;
import javax.servlet.ServletContext;

import com.espectaclia.entities.Descompte;
import com.espectaclia.entities.Espectacle;
import com.espectaclia.enumerations.TipusDescompte;
import com.espectaclia.daos.DescompteDAO;
import com.espectaclia.daos.EspectacleDAO;

import org.primefaces.event.SelectEvent;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.context.support.WebApplicationContextUtils;

@ManagedBean(name="descompteMB")
@ViewScoped
public class DescompteMB implements Serializable{

	private static final long serialVersionUID = 1L;
	private static final String messageCloseButton = "<button type=\"button\" class=\"close\" onclick=\"$(this).parent().fadeOut()\"><span aria-hidden=\"true\">×</span></button>";
	
	@Autowired
	private DescompteDAO descompteDAO;
	
	@Autowired
	private EspectacleDAO espectacleDAO;

	private Espectacle espectacle;
	private Descompte descompte;
	private Descompte descompteSel;
	private Integer idRecinte;
	private Integer idEspectacle;
	private Integer idDescompteModificar;
	
	private Date currentDate = new Date();
	
	private List<Descompte> descomptes;
	private List<SelectItem> opcionsTipusDescompte;
	private List<SelectItem> opcionsTipusDescompteLlistat;
	
	@PostConstruct
	public void init() {
		// Importem l'Autowired (Spring) al ManagedBean (JSF) per poder injectar els DAOs
		ExternalContext externalContext = FacesContext.getCurrentInstance().getExternalContext();
        ServletContext servletContext = (ServletContext) externalContext.getContext();
        WebApplicationContextUtils.getRequiredWebApplicationContext(servletContext).getAutowireCapableBeanFactory().autowireBean(this);
		
        espectacle = new Espectacle();
		descompte = new Descompte();
		descompteSel = null;
		descomptes = new ArrayList<Descompte>();
		
		opcionsTipusDescompte = new ArrayList<SelectItem>();
	    opcionsTipusDescompte.add(new SelectItem(TipusDescompte.PACK_ENTRADES, "Pack entrades"));
	    opcionsTipusDescompte.add(new SelectItem(TipusDescompte.FAMILIA_NOMBROSA, "Família nombrosa"));
	    opcionsTipusDescompte.add(new SelectItem(TipusDescompte.SESSIO_GOLFA, "Sessió golfa"));
	    opcionsTipusDescompte.add(new SelectItem(TipusDescompte.SESSIO_ESPECIAL, "Sessió especial"));
	    opcionsTipusDescompte.add(new SelectItem(TipusDescompte.DIA_ESPECTADOR, "Dia espectador"));
	    opcionsTipusDescompte.add(new SelectItem(TipusDescompte.DISCAPACITAT, "Discapacitat"));
	    opcionsTipusDescompte.add(new SelectItem(TipusDescompte.CARNET_DESCOMPTE, "Carnet descompte"));
	    opcionsTipusDescompte.add(new SelectItem(TipusDescompte.INFANTIL, "Infantil"));
	    opcionsTipusDescompte.add(new SelectItem(TipusDescompte.JOVE, "Jove"));
	    opcionsTipusDescompte.add(new SelectItem(TipusDescompte.JUBILAT, "Jubilat"));
	    
	    opcionsTipusDescompteLlistat = new ArrayList<SelectItem>();
		opcionsTipusDescompteLlistat.add(new SelectItem("", "TOTS"));
		opcionsTipusDescompteLlistat.addAll(opcionsTipusDescompte);
    }
	
	public void loadEspectacle() {
		try {
			if (this.idEspectacle != null) {
				this.espectacle = espectacleDAO.cercarPerId(this.idEspectacle);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public void loadDescomptes() {
		this.descompte = new Descompte();
		this.descomptes = new ArrayList<Descompte>();
		
		try {
			this.descomptes.addAll(descompteDAO.cercarPerPropietat("espectacle", this.espectacle));
			
			// Si es tracta d'una modificació, busquem la funció a modificar
			if (this.idDescompteModificar != null) {
				for (Descompte des : this.descomptes) {
					if (des.getId() == this.idDescompteModificar) {
						this.descompte = des;
						break;
					}
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public String crear() {
		try {
			// Comprovem si ja existeix un altre descompte amb el mateix nom pel mateix espectacle
			if (this.descomptes.size() > 0) {
				for (Descompte descompteObj : this.descomptes) {
					if (descompteObj.getNom().equals(descompte.getNom())) {
						UIInput uiInputUsuari = (UIInput) FacesContext.getCurrentInstance().getViewRoot().findComponent("descompte_form:descompte_nom_input");
						uiInputUsuari.setValid(false);
						
						FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "L'espectacle "+espectacle.getNom()+" ja té un descompte amb el nom \""+descompte.getNom()+"\". Tria un altre nom." + messageCloseButton, "");
			            FacesContext.getCurrentInstance().addMessage(null, message);
			            return null;
					}
				}
			}
			
			descompte.setEspectacle(espectacle);
			
			descompteDAO.persistir(this.descompte);

			// El nou descompte s'ha creat amb èxit
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_INFO, "El descompte s'ha creat amb èxit." + messageCloseButton, "");
			FacesContext.getCurrentInstance().addMessage(null, message);
			FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);
            return tornarGestioDescomptes();
		} catch (Exception e) {
			// S'ha produït un error desconegut intentant crear el nou descompte
			e.printStackTrace();
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "No s'ha pogut crear el descompte. Si us plau, comprova que les dades introduïdes siguin correctes." + messageCloseButton, "");
            FacesContext.getCurrentInstance().addMessage(null, message);
            return null;
		}
	}
	
	public String guardar() {
		try {
			// Comprovem si ja existeix un altre descompte amb el mateix nom pel mateix espectacle
			if (this.descomptes.size() > 0) {
				for (Descompte descompteObj : this.descomptes) {
					if (descompteObj.getNom().equals(descompte.getNom()) && descompteObj.getId() != descompte.getId()) {
						UIInput uiInputUsuari = (UIInput) FacesContext.getCurrentInstance().getViewRoot().findComponent("descompte_form:descompte_nom_input");
						uiInputUsuari.setValid(false);
						
						FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "L'espectacle "+espectacle.getNom()+" ja té un descompte amb el nom \""+descompte.getNom()+"\". Tria un altre nom." + messageCloseButton, "");
			            FacesContext.getCurrentInstance().addMessage(null, message);
			            return null;
					}
				}
			}
			
			descompteDAO.unir(this.descompte);

			// El descompte s'ha modificat amb èxit
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_INFO, "El descompte s'ha modificat amb èxit." + messageCloseButton, "");
			FacesContext.getCurrentInstance().addMessage(null, message);
			FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);
			return tornarGestioDescomptes();
		} catch (Exception e) {
			// S'ha produït un error desconegut intentant modificar el descompte
			e.printStackTrace();
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "No s'ha pogut modificar el descompte. Si us plau, comprova que les dades modificades siguin correctes." + messageCloseButton, "");
            FacesContext.getCurrentInstance().addMessage(null, message);
            return null;
		}
	}
	
	public String eliminar(Descompte des) {
		try {
			// Eliminem el descompte de la llista de descomptes de l'espectacle
			if (espectacle != null) {
				for (Descompte descompteObj : espectacle.getDescomptes()) {
					if (descompteObj.getId() == des.getId()) {
						espectacle.getDescomptes().remove(descompteObj);
						espectacleDAO.unir(espectacle);
						break;
					}
				}
			}
			
			// El descompte s'ha eliminat amb èxit
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_INFO, "El descompte s'ha eliminat amb èxit." + messageCloseButton, "");
			FacesContext.getCurrentInstance().addMessage(null, message);
			FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);
			return tornarGestioDescomptes();
		} catch (Exception e) {
			// S'ha produït un error desconegut intentant eliminar el descompte
			e.printStackTrace();
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "S'ha produït un error i no s'ha pogut eliminar el descompte." + messageCloseButton, "");
            FacesContext.getCurrentInstance().addMessage(null, message);
		}
		return null;
	}
	
	public void seleccionarDescompte(SelectEvent event) throws IOException {
		descompteSel = (Descompte) event.getObject();
		
		ConfigurableNavigationHandler configurableNavigationHandler = (ConfigurableNavigationHandler) FacesContext.getCurrentInstance().getApplication().getNavigationHandler();
		configurableNavigationHandler.performNavigation("/views/app/inici.xhtml?faces-redirect=true&idDescompte="+descompteSel.getId());
	}
	
	public String tornarGestioEspectacles() {
		return "/views/app/espectacle/espectacles.xhtml?faces-redirect=true&idRecinte="+idRecinte;
	}
	
	public String tornarGestioDescomptes() {
		return "/views/app/descompte/descomptes.xhtml?faces-redirect=true&idRecinte="+idRecinte+"&idEspectacle="+idEspectacle;
	}
	
	public String nouDescompte() {
		return "/views/app/descompte/modificar_descompte.xhtml?faces-redirect=true&idRecinte="+idRecinte+"&idEspectacle="+idEspectacle;
	}
	
	public String modificarDescompte(Integer id) {
		return "/views/app/descompte/modificar_descompte.xhtml?faces-redirect=true&idRecinte="+idRecinte+"&idEspectacle="+idEspectacle+"&idDescompteModificar="+id;
	}
	
	public List<Object> getBreadcrumbParams() {
		List<Object> params = new ArrayList<Object>();
		params.add(idRecinte);
		params.add(idEspectacle);
		params.add(idDescompteModificar);
		
		return params;
	}
	
	public String getMessageCloseButton() {
		return messageCloseButton;
	}
	
	public Espectacle getEspectacle() {
		return espectacle;
	}
	
	public void setEspectacle(Espectacle espectacle) {
		this.espectacle = espectacle;
	}
	
	public Descompte getDescompte() {
		return descompte;
	}
	
	public void setDescompte(Descompte descompte) {
		this.descompte = descompte;
	}
	
	public Descompte getDescompteSel() {
		return descompteSel;
	}
	
	public void setDescompteSel(Descompte descompteSel) {
		this.descompteSel = descompteSel;
	}
	
	public Integer getIdRecinte() {
		return idRecinte;
	}
	
	public void setIdRecinte(Integer idRecinte) {
		this.idRecinte = idRecinte;
	}
	
	public Integer getIdEspectacle() {
		return idEspectacle;
	}
	
	public void setIdEspectacle(Integer idEspectacle) {
		this.idEspectacle = idEspectacle;
	}
	
	public Integer getIdDescompteModificar() {
		return idDescompteModificar;
	}
	
	public void setIdDescompteModificar(Integer idDescompteModificar) {
		this.idDescompteModificar = idDescompteModificar;
	}
	
	public Date getCurrentDate() {
		return currentDate;
	}
	
	public void setCurrentDate(Date currentDate) {
		this.currentDate = currentDate;
	}
	
	public List<Descompte> getDescomptes() {
		return descomptes;
	}
	
	public void setDescomptes(List<Descompte> descomptes) {
		this.descomptes = descomptes;
	}
	
	public List<SelectItem> getOpcionsTipusDescompte() {
		return opcionsTipusDescompte;
	}
	
	public void setOpcionsTipusDescompte(List<SelectItem> opcionsTipusDescompte) {
		this.opcionsTipusDescompte = opcionsTipusDescompte;
	}
	
	public List<SelectItem> getOpcionsTipusDescompteLlistat() {
		return opcionsTipusDescompteLlistat;
	}
	
	public void setOpcionsTipusDescompteLlistat(List<SelectItem> opcionsTipusDescompteLlistat) {
		this.opcionsTipusDescompteLlistat = opcionsTipusDescompteLlistat;
	}

}