package com.espectaclia.managed_beans;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ViewScoped;
import javax.faces.component.UIInput;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.faces.model.SelectItem;
import javax.servlet.ServletContext;

import com.espectaclia.entities.Espectacle;
import com.espectaclia.entities.Recinte;
import com.espectaclia.enumerations.TipusEspectacle;
import com.espectaclia.daos.EspectacleDAO;
import com.espectaclia.daos.RecinteDAO;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.context.support.WebApplicationContextUtils;

@ManagedBean(name="espectacleMB")
@ViewScoped
public class EspectacleMB implements Serializable{

	private static final long serialVersionUID = 1L;
	private static final String messageCloseButton = "<button type=\"button\" class=\"close\" onclick=\"$(this).parent().fadeOut()\"><span aria-hidden=\"true\">×</span></button>";
	
	@Autowired
	private EspectacleDAO espectacleDAO;
	
	@Autowired
	private RecinteDAO recinteDAO;

	private Recinte recinte;
	private Espectacle espectacle;
	private Integer idRecinte;
	private Integer idEspectacleModificar;
	
	private List<Espectacle> espectacles;
	private List<SelectItem> opcionsTipusEspectacle;
	private List<SelectItem> opcionsTipusEspectacleLlistat;
	
	@PostConstruct
	public void init() {
		// Importem l'Autowired (Spring) al ManagedBean (JSF) per poder injectar els DAOs
		ExternalContext externalContext = FacesContext.getCurrentInstance().getExternalContext();
        ServletContext servletContext = (ServletContext) externalContext.getContext();
        WebApplicationContextUtils.getRequiredWebApplicationContext(servletContext).getAutowireCapableBeanFactory().autowireBean(this);
		
        recinte = new Recinte();
		espectacle = new Espectacle();
		espectacles = new ArrayList<Espectacle>();
		
		opcionsTipusEspectacle = new ArrayList<SelectItem>();
	    opcionsTipusEspectacle.add(new SelectItem(TipusEspectacle.OBRA_TEATRAL, "Obra teatral"));
	    opcionsTipusEspectacle.add(new SelectItem(TipusEspectacle.DANSA, "Dansa"));
	    opcionsTipusEspectacle.add(new SelectItem(TipusEspectacle.PELLICULA, "Pel·lícula"));
	    opcionsTipusEspectacle.add(new SelectItem(TipusEspectacle.EXPOSICIO, "Exposició"));
	    opcionsTipusEspectacle.add(new SelectItem(TipusEspectacle.CONCERT, "Concert"));
	    opcionsTipusEspectacle.add(new SelectItem(TipusEspectacle.CONFERENCIA, "Conferència"));
	    opcionsTipusEspectacle.add(new SelectItem(TipusEspectacle.ESDEVENIMENT_ESPORTIU, "Esdev. esportiu"));
	    opcionsTipusEspectacle.add(new SelectItem(TipusEspectacle.CONGRES, "Congrés"));
	    opcionsTipusEspectacle.add(new SelectItem(TipusEspectacle.ALTRES, "Altres"));
	    
	    opcionsTipusEspectacleLlistat = new ArrayList<SelectItem>();
		opcionsTipusEspectacleLlistat.add(new SelectItem("", "TOTS"));
		opcionsTipusEspectacleLlistat.addAll(opcionsTipusEspectacle);
    }
	
	public void loadRecinte() {
		try {
			if (this.idRecinte != null) {
				this.recinte = recinteDAO.cercarPerId(this.idRecinte);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public void loadEspectacles() {
		this.espectacle = new Espectacle();
		this.espectacles = new ArrayList<Espectacle>();
		
		try {
			this.espectacles.addAll(espectacleDAO.cercarPerPropietat("recinte", this.recinte));
			
			// Si es tracta d'una modificació, busquem el recinte a modificar
			if (this.idEspectacleModificar != null) {
				for (Espectacle esp : this.espectacles) {
					if (esp.getId() == this.idEspectacleModificar) {
						this.espectacle = esp;
						break;
					}
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public String crear() {
		try {
			// Comprovem si ja existeix un altre espectacle amb el mateix nom del mateix recinte
			if (this.espectacles.size() > 0) {
				for (Espectacle espectacleObj : this.espectacles) {
					if (espectacleObj.getNom().equalsIgnoreCase(espectacle.getNom())) {
						UIInput uiInputUsuari = (UIInput) FacesContext.getCurrentInstance().getViewRoot().findComponent("espectacle_form:espectacle_nom_input");
						uiInputUsuari.setValid(false);
						
						FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "El recinte "+recinte.getNom()+" ja té un espectacle amb el nom \""+espectacle.getNom()+"\". Tria un altre nom." + messageCloseButton, "");
			            FacesContext.getCurrentInstance().addMessage(null, message);
			            return null;
					}
				}
			}
			
			espectacle.setRecinte(recinte);
			
			espectacleDAO.persistir(this.espectacle);

			// El nou espectacle s'ha creat amb èxit
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_INFO, "L'espectacle s'ha creat amb èxit." + messageCloseButton, "");
			FacesContext.getCurrentInstance().addMessage(null, message);
			FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);
            return tornarGestioEspectacles();
		} catch (Exception e) {
			// S'ha produït un error desconegut intentant crear el nou espectacle
			e.printStackTrace();
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "No s'ha pogut crear l'espectacle. Si us plau, comprova que les dades introduïdes siguin correctes." + messageCloseButton, "");
            FacesContext.getCurrentInstance().addMessage(null, message);
            return null;
		}
	}
	
	public String guardar() {
		try {
			// Comprovem si ja existeix un altre recinte amb el mateix nom del mateix recinte
			if (this.espectacles.size() > 0) {
				for (Espectacle espectacleObj : this.espectacles) {
					if (espectacleObj.getNom().equalsIgnoreCase(espectacle.getNom()) && espectacleObj.getId() != espectacle.getId()) {
						UIInput uiInputUsuari = (UIInput) FacesContext.getCurrentInstance().getViewRoot().findComponent("espectacle_form:espectacle_nom_input");
						uiInputUsuari.setValid(false);
						
						FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "El recinte "+recinte.getNom()+" ja té un espectacle amb el nom \""+espectacle.getNom()+"\". Tria un altre nom." + messageCloseButton, "");
			            FacesContext.getCurrentInstance().addMessage(null, message);
			            return null;
					}
				}
			}
			
			espectacleDAO.unir(this.espectacle);

			// L'espectacle s'ha modificat amb èxit
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_INFO, "L'espectacle s'ha modificat amb èxit." + messageCloseButton, "");
			FacesContext.getCurrentInstance().addMessage(null, message);
			FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);
			return tornarGestioEspectacles();
		} catch (Exception e) {
			// S'ha produït un error desconegut intentant modificar l'espectacle
			e.printStackTrace();
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "No s'ha pogut modificar l'espectacle. Si us plau, comprova que les dades modificades siguin correctes." + messageCloseButton, "");
            FacesContext.getCurrentInstance().addMessage(null, message);
            return null;
		}
	}
	
	public String eliminar(Espectacle esp) {
		try {
			// Comprovem que l'espectacle no tingui cap funció activa
			if (esp.teFuncionsActives()) {
				FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "L'espectacle no es pot eliminar perquè té funcions programades actives. Cancel·la o elimina primer totes les funcions." + messageCloseButton, "");
	            FacesContext.getCurrentInstance().addMessage(null, message);
	            return null;
			}
			
			// Eliminem l'espectacle de la llista d'espectacles del recinte
			if (this.recinte != null) {
				for (Espectacle espectacleObj : this.recinte.getEspectacles()) {
					if (espectacleObj.getId() == esp.getId()) {
						this.recinte.getEspectacles().remove(espectacleObj);
						recinteDAO.unir(this.recinte);
						break;
					}
				}
			}
			
			// L'espectacle s'ha eliminat amb èxit
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_INFO, "L'espectacle s'ha eliminat amb èxit." + messageCloseButton, "");
			FacesContext.getCurrentInstance().addMessage(null, message);
			FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);
			return tornarGestioEspectacles();
		} catch (Exception e) {
			// S'ha produït un error desconegut intentant eliminar l'espectacle
			e.printStackTrace();
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "S'ha produït un error i no s'ha pogut eliminar l'espectacle." + messageCloseButton, "");
            FacesContext.getCurrentInstance().addMessage(null, message);
		}
		return null;
	}
	
	public String tornarGestioEspectacles() {
		return "/views/app/espectacle/espectacles.xhtml?faces-redirect=true&idRecinte="+idRecinte;
	}
	
	public String nouEspectacle() {
		return "/views/app/espectacle/modificar_espectacle.xhtml?faces-redirect=true&idRecinte="+idRecinte;
	}
	
	public String modificarEspectacle(Integer id) {
		return "/views/app/espectacle/modificar_espectacle.xhtml?faces-redirect=true&idRecinte="+idRecinte+"&idEspectacleModificar="+id;
	}
	
	public String gestionarFuncions(Integer id) {
		return "/views/app/funcio/funcions.xhtml?faces-redirect=true&idRecinte="+idRecinte+"&idEspectacle="+id;
	}
	
	public String gestionarPreusLocalitats(Integer id) {
		return "/views/app/preuslocs/preuslocs.xhtml?faces-redirect=true&idRecinte="+idRecinte+"&idEspectacle="+id;
	}
	
	public String gestionarDescomptes(Integer id) {
		return "/views/app/descompte/descomptes.xhtml?faces-redirect=true&idRecinte="+idRecinte+"&idEspectacle="+id;
	}
	
	public List<Object> getBreadcrumbParams() {
		List<Object> params = new ArrayList<Object>();
		params.add(idRecinte);
		params.add(idEspectacleModificar);
		
		return params;
	}
	
	public String getMessageCloseButton() {
		return messageCloseButton;
	}
	
	public Recinte getRecinte() {
		return recinte;
	}
	
	public void setRecinte(Recinte recinte) {
		this.recinte = recinte;
	}
	
	public Espectacle getEspectacle() {
		return espectacle;
	}
	
	public void setEspectacle(Espectacle espectacle) {
		this.espectacle = espectacle;
	}
	
	public Integer getIdRecinte() {
		return idRecinte;
	}
	
	public void setIdRecinte(Integer idRecinte) {
		this.idRecinte = idRecinte;
	}
	
	public Integer getIdEspectacleModificar() {
		return idEspectacleModificar;
	}
	
	public void setIdEspectacleModificar(Integer idEspectacleModificar) {
		this.idEspectacleModificar = idEspectacleModificar;
	}
	
	public List<Espectacle> getEspectacles() {
		return espectacles;
	}
	
	public void setEspectacles(List<Espectacle> espectacles) {
		this.espectacles = espectacles;
	}
	
	public List<SelectItem> getOpcionsTipusEspectacle() {
		return opcionsTipusEspectacle;
	}
	
	public void setOpcionsTipusEspectacle(List<SelectItem> opcionsTipusEspectacle) {
		this.opcionsTipusEspectacle = opcionsTipusEspectacle;
	}
	
	public List<SelectItem> getOpcionsTipusEspectacleLlistat() {
		return opcionsTipusEspectacleLlistat;
	}
	
	public void setOpcionsTipusEspectacleLlistat(List<SelectItem> opcionsTipusEspectacleLlistat) {
		this.opcionsTipusEspectacleLlistat = opcionsTipusEspectacleLlistat;
	}

}