package com.espectaclia.managed_beans;

import java.io.IOException;
import java.io.Serializable;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.faces.application.ConfigurableNavigationHandler;
import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ViewScoped;
import javax.faces.component.UIInput;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.faces.model.SelectItem;
import javax.servlet.ServletContext;

import com.espectaclia.entities.Client;
import com.espectaclia.entities.Funcio;
import com.espectaclia.entities.Espectacle;
import com.espectaclia.entities.Localitat;
import com.espectaclia.entities.Reserva;
import com.espectaclia.entities.Zona;
import com.espectaclia.enumerations.EstatFuncio;
import com.espectaclia.enumerations.EstatReserva;
import com.espectaclia.utils.RecinteRenderer;
import com.espectaclia.daos.ClientDAO;
import com.espectaclia.daos.FuncioDAO;
import com.espectaclia.daos.EspectacleDAO;
import com.espectaclia.daos.LocalitatDAO;
import com.espectaclia.daos.ReservaDAO;
import com.espectaclia.daos.ZonaDAO;

import org.primefaces.event.SelectEvent;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.context.support.WebApplicationContextUtils;

@ManagedBean(name="funcioMB")
@ViewScoped
public class FuncioMB implements Serializable{

	private static final long serialVersionUID = 1L;
	private static final String messageCloseButton = "<button type=\"button\" class=\"close\" onclick=\"$(this).parent().fadeOut()\"><span aria-hidden=\"true\">×</span></button>";
	
	@Autowired
	private FuncioDAO funcioDAO;
	
	@Autowired
	private EspectacleDAO espectacleDAO;
	
	@Autowired
	private ZonaDAO zonaDAO;
	
	@Autowired
	private ReservaDAO reservaDAO;
	
	@Autowired
	private LocalitatDAO localitatDAO;
	
	@Autowired
	private ClientDAO clientDAO;
	
	@Autowired
	private RecinteRenderer recinteRenderer;

	private Espectacle espectacle;
	private Funcio funcio;
	private Funcio funcioSel;
	private Integer idRecinte;
	private Integer idEspectacle;
	private Integer idFuncioModificar;
	
	private Date dataOriginal;
	private Date currentDate = new Date();
	
	private List<Funcio> funcions;
	private List<SelectItem> opcionsEstatFuncio;
	private List<SelectItem> opcionsEstatFuncioLlistat;
	
	private List<Zona> zones;
	private List<Localitat> localitatsBloquejades = new ArrayList<Localitat>();
	private List<Localitat> localitatsDesbloquejades = new ArrayList<Localitat>();
	
	@PostConstruct
	public void init() {
		// Importem l'Autowired (Spring) al ManagedBean (JSF) per poder injectar els DAOs
		ExternalContext externalContext = FacesContext.getCurrentInstance().getExternalContext();
        ServletContext servletContext = (ServletContext) externalContext.getContext();
        WebApplicationContextUtils.getRequiredWebApplicationContext(servletContext).getAutowireCapableBeanFactory().autowireBean(this);
		
        espectacle = new Espectacle();
		funcio = new Funcio();
		funcioSel = null;
		funcions = new ArrayList<Funcio>();
		
		opcionsEstatFuncio = new ArrayList<SelectItem>();
	    opcionsEstatFuncio.add(new SelectItem(EstatFuncio.ACTIVA, "Activa"));
	    opcionsEstatFuncio.add(new SelectItem(EstatFuncio.CANCELLADA, "Cancel·lada"));
	    
	    opcionsEstatFuncioLlistat = new ArrayList<SelectItem>();
		opcionsEstatFuncioLlistat.add(new SelectItem("", "TOTS"));
		opcionsEstatFuncioLlistat.addAll(opcionsEstatFuncio);
    }
	
	public void loadEspectacle() {
		try {
			if (this.idEspectacle != null) {
				this.espectacle = espectacleDAO.cercarPerId(this.idEspectacle);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public void loadFuncions() {
		this.funcio = new Funcio();
		this.funcions = new ArrayList<Funcio>();
		
		try {
			this.funcions.addAll(funcioDAO.cercarPerPropietat("espectacle", this.espectacle));
			
			// Si es tracta d'una modificació, busquem la funció a modificar
			if (this.idFuncioModificar != null) {
				for (Funcio fun : this.funcions) {
					if (fun.getId() == this.idFuncioModificar) {
						this.funcio = fun;
						dataOriginal = fun.getData();
						break;
					}
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public void loadZones() {
		this.zones = new ArrayList<Zona>();
		
		try {
			this.zones.addAll(zonaDAO.cercarPerPropietat("recinte", this.espectacle.getRecinte()));
			
			// Generem les localitats per pintar el pati de butaques de les zones amb localitats numerades
			for (Zona zon : this.zones) {
				if (zon.getNumFiles() != null && zon.getNumSeientsFila() != null) {
					List<Localitat> localitats = new ArrayList<Localitat>();
					
					for (int numFila=1; numFila<=zon.getNumFiles(); numFila++) {
						for (int numSeient=1; numSeient<=zon.getNumSeientsFila(); numSeient++) {
							Localitat loc = new Localitat();
							loc.setFila(numFila);
							loc.setSeient(numSeient);
							loc.setZona(zon);
							
							localitats.add(loc);
						}
					}
					
					zon.setLocalitatsPatiButaques(localitats);
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public void loadLocalitatsBloquejades() {
		if (this.funcio != null) {
			for (Reserva reserva : this.funcio.getReserves()) {
				if (reserva.getEstat() == EstatReserva.BLOQUEJADA) {
					for (Localitat localitat : reserva.getLocalitats()) {
						if (!isLocalitatDesbloquejada(localitat) && !isLocalitatBloquejada(localitat)) {
							this.localitatsBloquejades.add(localitat);
						}
					}
				}
			}
		}
	}
	
	public String crear() {
		try {
			// Comprovem si ja existeix una altre funcio amb la mateixa data i hora pel mateix espectacle
			if (this.funcions.size() > 0) {
				for (Funcio funcioObj : this.funcions) {
					SimpleDateFormat sdf = new SimpleDateFormat("dd-MM-yyyy HH:mm");
					if (sdf.format(funcioObj.getData()).equals(sdf.format(funcio.getData()))) {
						UIInput uiInputUsuari = (UIInput) FacesContext.getCurrentInstance().getViewRoot().findComponent("funcio_form:funcio_data_input");
						uiInputUsuari.setValid(false);
						
						FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "L'espectacle "+espectacle.getNom()+" ja té una funció programada en aquesta data i hora. Tria una altra data i/o una altra hora." + messageCloseButton, "");
			            FacesContext.getCurrentInstance().addMessage(null, message);
			            return null;
					}
				}
			}
			
			funcio.setEspectacle(this.espectacle);
			
			funcioDAO.persistir(this.funcio);

			// La nova funció s'ha creat amb èxit
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_INFO, "La funció s'ha creat amb èxit." + messageCloseButton, "");
			FacesContext.getCurrentInstance().addMessage(null, message);
			FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);
            return tornarGestioFuncions();
		} catch (Exception e) {
			// S'ha produït un error desconegut intentant crear la nova funció
			e.printStackTrace();
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "No s'ha pogut crear la funció. Si us plau, comprova que les dades introduïdes siguin correctes." + messageCloseButton, "");
            FacesContext.getCurrentInstance().addMessage(null, message);
            return null;
		}
	}
	
	public String guardar() {
		try {
			// Si la funció té estat 'Activa', ha cambiat la data i té reserves pendents o venudes, s'avisa l'usuari que no es pot modificar la data
			if (this.funcio.getEstat() == EstatFuncio.ACTIVA && !this.funcio.getData().equals(this.dataOriginal) && (this.funcio.teReservesPendents() || this.funcio.teReservesVenudes())) {
				FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "No es pot modificar la data d'aquesta funció perquè té reserves pendents i/o venudes." + messageCloseButton, "");
	            FacesContext.getCurrentInstance().addMessage(null, message);
	            return null;
			}
			
			// Si la funció té estat 'Cancel·lada' i té reserves venudes, s'avisa l'usuari que no es pot cancel·lar
			if (this.funcio.getEstat() == EstatFuncio.CANCELLADA && this.funcio.teReservesVenudes()) {
				FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "No es pot cancel·lar aquesta funció perquè ja té reserves venudes." + messageCloseButton, "");
	            FacesContext.getCurrentInstance().addMessage(null, message);
	            return null;
			}
			
			// Si la funció té estat 'Cancel·lada' i té reserves pendents, es cancel·len totes aquestes reserves
			if (this.funcio.getEstat() == EstatFuncio.CANCELLADA && this.funcio.teReservesPendents()) {
				for (Reserva reserva : this.funcio.getReserves()) {
					if (reserva.getEstat() == EstatReserva.CONFIRMADA) {
						reserva.setEstat(EstatReserva.CANCELLADA);
						reserva.setDataUltModificacio(new Date());
						
						reservaDAO.unir(reserva);
					}
				}
			}
			
			// Comprovem si ja existeix una altre funcio amb la mateixa data i hora pel mateix espectacle
			if (this.funcions.size() > 0) {
				for (Funcio funcioObj : this.funcions) {
					SimpleDateFormat sdf = new SimpleDateFormat("dd-MM-yyyy HH:mm");
					if (sdf.format(funcioObj.getData()).equals(sdf.format(funcio.getData())) && funcioObj.getId() != funcio.getId()) {
						UIInput uiInputUsuari = (UIInput) FacesContext.getCurrentInstance().getViewRoot().findComponent("funcio_form:funcio_data_input");
						uiInputUsuari.setValid(false);
						
						FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "L'espectacle "+espectacle.getNom()+" ja té una funció programada en aquesta data i hora. Tria una altra data i/o una altra hora." + messageCloseButton, "");
			            FacesContext.getCurrentInstance().addMessage(null, message);
			            return null;
					}
				}
			}
			
			funcioDAO.unir(this.funcio);

			// La funció s'ha modificat / cancel·lat amb èxit
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_INFO, "La funció s'ha "+(this.funcio.getEstat() == EstatFuncio.CANCELLADA ? "cancel·lat" : "modificat")+" amb èxit." + messageCloseButton, "");
			FacesContext.getCurrentInstance().addMessage(null, message);
			FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);
			return tornarGestioFuncions();
		} catch (Exception e) {
			// S'ha produït un error desconegut intentant modificar la funció
			e.printStackTrace();
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "No s'ha pogut modificar la funció. Si us plau, comprova que les dades modificades siguin correctes." + messageCloseButton, "");
            FacesContext.getCurrentInstance().addMessage(null, message);
            return null;
		}
	}
	
	public String eliminar(Funcio fun) {
		try {
			// Comprovem que la funció no tingui cap reserva pendent (Estat: CONFIRMADA)
			if (fun.teReservesPendents()) {
				FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "La funció no es pot eliminar perquè té reserves pendents." + messageCloseButton, "");
	            FacesContext.getCurrentInstance().addMessage(null, message);
	            return null;
			}
			
			// Eliminem la funció de la llista de funcions de l'espectacle
			if (this.espectacle != null) {
				for (Funcio funcioObj : this.espectacle.getFuncions()) {
					if (funcioObj.getId() == fun.getId()) {
						espectacle.getFuncions().remove(funcioObj);
						espectacleDAO.unir(espectacle);
						break;
					}
				}
			}
			
			// La funció s'ha eliminat amb èxit
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_INFO, "La funció s'ha eliminat amb èxit." + messageCloseButton, "");
			FacesContext.getCurrentInstance().addMessage(null, message);
			FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);
			return tornarGestioFuncions();
		} catch (Exception e) {
			// S'ha produït un error desconegut intentant eliminar la funció
			e.printStackTrace();
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "S'ha produït un error i no s'ha pogut eliminar la funció." + messageCloseButton, "");
            FacesContext.getCurrentInstance().addMessage(null, message);
		}
		return null;
	}
	
	public String confirmarLocsBloquejades() {
		try {
			// Buidem totes les reserves de localitats bloquejades de la funció
			List<Reserva> reservesBloquejades = new ArrayList<Reserva>();
			for(Reserva reserva : this.funcio.getReserves()) {
				if (reserva.getEstat() == EstatReserva.BLOQUEJADA) {
					Zona zona = reserva.getZona();
					zona.getLocalitats().removeAll(reserva.getLocalitats());
					zona.getReserves().remove(reserva);
			
					Client client = reserva.getClient();
					client.getReserves().remove(reserva);
					
					reservesBloquejades.add(reserva);
					
					reservaDAO.eliminar(reserva);
				}
			}
			List<Reserva> reserves = this.funcio.getReserves();
			reserves.removeAll(reservesBloquejades);
			
			List<Reserva> reservesNet = new ArrayList<Reserva>();
			reservesNet.addAll(reserves);
			
			this.funcio.setReserves(reservesNet);
			
			reservesBloquejades.clear();
			
			// Per a cada localitat bloquejada
			for (Localitat locBloquejada : this.localitatsBloquejades) {
				locBloquejada.setReserva(null);
				
				boolean existeixReservaBloqueig = false;
				// Per a cada reserva de la funció, cerquem les reserves de localitats bloquejades
				for (Reserva reserva : this.funcio.getReserves()) {
					// Si ja existeix una reserva de localitats bloquejades de la mateixa zona que la localitat bloquejada actual
					if (reserva.getEstat() == EstatReserva.BLOQUEJADA && reserva.getZona().getId() == locBloquejada.getZona().getId()) {
						boolean localitatTrobada = false;
						// Cerquem si ja existeix la localitat bloquejada en aquesta reserva, i si no hi és l'afegim
						for (Localitat loc : reserva.getLocalitats()) {
							if (loc.getFila() == locBloquejada.getFila() && loc.getSeient() == locBloquejada.getSeient()) {
								localitatTrobada = true;
								break;
							}
						}
						
						if (!localitatTrobada) {
							locBloquejada.setReserva(reserva);
							
							reserva.getLocalitats().add(locBloquejada);
							reserva.setNumLocalitats(reserva.getLocalitats().size());
							reserva.setDataUltModificacio(new Date());
						}
						
						existeixReservaBloqueig = true;
						break;
					}
				}
				
				// Si no existeix una reserva de localitats bloquejades de la mateixa zona que la localitat bloquejada actual, creem una nova reserva
				if (!existeixReservaBloqueig) {
					List<Localitat> localitats = new ArrayList<Localitat>();
					localitats.add(locBloquejada);
					
					// Obtenim el Client especial per bloquejar localitats (no es pot fer login amb ell)
					Client clientBloqueig = clientDAO.cercarPerId(9999);
					
					Reserva novaReserva = new Reserva();
					novaReserva.setFuncio(this.funcio);
					novaReserva.setZona(locBloquejada.getZona());
					novaReserva.setClient(clientBloqueig);
					novaReserva.setCodiQR("");
					novaReserva.setPreuFinal(0);
					novaReserva.setEstat(EstatReserva.BLOQUEJADA);
					novaReserva.setLocalitats(localitats);
					novaReserva.setNumLocalitats(1);
					novaReserva.setDataCreacio(new Date());
					
					locBloquejada.setReserva(novaReserva);
					
					this.funcio.getReserves().add(novaReserva);
				}
			}
			
			funcioDAO.unir(this.funcio);
			
			// Les localitats bloquejades s'han modificat amb èxit
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_INFO, "Les localitats bloquejades s'han modificat amb èxit." + messageCloseButton, "");
			FacesContext.getCurrentInstance().addMessage(null, message);
			FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);
			return tornarGestioFuncions();
		} catch (Exception e) {
			// S'ha produït un error desconegut intentant modificar les localitats bloquejades
			e.printStackTrace();
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "S'ha produït un error i no s'han pogut modificar les localitats bloquejades." + messageCloseButton, "");
            FacesContext.getCurrentInstance().addMessage(null, message);
		}
		
		return null;
	}
	
	public void seleccionarFuncio(SelectEvent event) throws IOException {
		funcioSel = (Funcio) event.getObject();
		
		ConfigurableNavigationHandler configurableNavigationHandler = (ConfigurableNavigationHandler) FacesContext.getCurrentInstance().getApplication().getNavigationHandler();
		configurableNavigationHandler.performNavigation("/views/app/inici.xhtml?faces-redirect=true&idFuncio="+funcioSel.getId());
	}
	
	public String tornarGestioEspectacles() {
		return "/views/app/espectacle/espectacles.xhtml?faces-redirect=true&idRecinte="+idRecinte;
	}
	
	public String tornarGestioFuncions() {
		return "/views/app/funcio/funcions.xhtml?faces-redirect=true&idRecinte="+idRecinte+"&idEspectacle="+idEspectacle;
	}
	
	public String novaFuncio() {
		return "/views/app/funcio/modificar_funcio.xhtml?faces-redirect=true&idRecinte="+idRecinte+"&idEspectacle="+idEspectacle;
	}
	
	public String modificarFuncio(Integer id) {
		return "/views/app/funcio/modificar_funcio.xhtml?faces-redirect=true&idRecinte="+idRecinte+"&idEspectacle="+idEspectacle+"&idFuncioModificar="+id;
	}
	
	public String bloquejarLocalitats(Integer id) {
		return "/views/app/funcio/bloquejar_locs.xhtml?faces-redirect=true&idRecinte="+idRecinte+"&idEspectacle="+idEspectacle+"&idFuncioModificar="+id;
	}
	
	public List<Object> getBreadcrumbParams() {
		List<Object> params = new ArrayList<Object>();
		params.add(idRecinte);
		params.add(idEspectacle);
		params.add(idFuncioModificar);
		
		return params;
	}
	
	public String getZonaStyle(Zona zona, int ampladaRecinte, int alturaRecinte, int margeLateral) {
		if (zona.getCoordInici() != null && zona.getCoordInici().trim().length() > 0 && zona.getCoordFi() != null && zona.getCoordFi().trim().length() > 0) {
			double left = recinteRenderer.getZonaLeft(zona, ampladaRecinte, margeLateral);
			double top = recinteRenderer.getZonaTop(zona, alturaRecinte, margeLateral);
			double width = recinteRenderer.getZonaWidth(zona, ampladaRecinte, margeLateral);
			double height = recinteRenderer.getZonaHeight(zona, alturaRecinte, margeLateral);
			
			return "left: "+left+"px;"+
				"top: "+top+"px;"+
				"width: "+width+"px;"+
				"height: "+height+"px;";
		} else {
			return "display: none";
		}
	}
	
	public String getLocalitatZonaStyle(Localitat localitat, Zona zona, int ampladaRecinte, int alturaRecinte, int margeLateralZona, int margeLateralLocalitat) {
		double left = recinteRenderer.getLocalitatZonaLeft(localitat, zona, ampladaRecinte, margeLateralZona, margeLateralLocalitat);
		double top = recinteRenderer.getLocalitatZonaTop(localitat, zona, alturaRecinte, margeLateralZona, margeLateralLocalitat);
		double width = recinteRenderer.getLocalitatZonaWidth(localitat, zona, ampladaRecinte, margeLateralZona, margeLateralLocalitat);
		double height = recinteRenderer.getLocalitatZonaHeight(localitat, zona, alturaRecinte, margeLateralZona, margeLateralLocalitat);
		
		return "left: "+left+"px;"+
				"top: "+top+"px;"+
				"width: "+width+"px;"+
				"height: "+height+"px;";
	}
	
	public String toggleLocalitat(Localitat loc) {
		// Si la localitat està reservada no fem res
		if (!isLocalitatReservada(loc)) {
			// Si la localitat està bloquejada, la desbloquegem
			if (isLocalitatBloquejada(loc)) {
				for (Localitat localitat : this.localitatsBloquejades) {
					if (localitat.getZona().getId() == loc.getZona().getId() && localitat.getFila() == loc.getFila() && localitat.getSeient() == loc.getSeient()) {
						this.localitatsBloquejades.remove(localitat);
						this.localitatsDesbloquejades.add(localitat);
						break;
					}
				}
			// Si la localitat no està bloquejada, la bloquegem
			} else {
				for (Localitat localitat : this.localitatsDesbloquejades) {
					if (localitat.getZona().getId() == loc.getZona().getId() && localitat.getFila() == loc.getFila() && localitat.getSeient() == loc.getSeient()) {
						this.localitatsDesbloquejades.remove(localitat);
						break;
					}
				}
				this.localitatsBloquejades.add(loc);
			}
		}
		
		return null;
	}
	
	public boolean isLocalitatReservada(Localitat loc) {
		for (Reserva reserva : this.funcio.getReserves()) {
			for (Localitat localitat : reserva.getLocalitats()) {
				if (localitat.getZona().getId() == loc.getZona().getId() && localitat.getFila() == loc.getFila() && localitat.getSeient() == loc.getSeient()) {
					return (reserva.getEstat() == EstatReserva.TEMPORAL || reserva.getEstat() == EstatReserva.CONFIRMADA || reserva.getEstat() == EstatReserva.VENUDA);
				}
			}
		}
		
		return false;
	}
	
	public boolean isLocalitatBloquejada(Localitat loc) {
		for (Localitat localitat : this.localitatsBloquejades) {
			if (localitat.getZona().getId() == loc.getZona().getId() && localitat.getFila() == loc.getFila() && localitat.getSeient() == loc.getSeient()) {
				return true;
			}
		}
		
		return false;
	}
	
	public boolean isLocalitatDesbloquejada(Localitat loc) {
		for (Localitat localitat : this.localitatsDesbloquejades) {
			if (localitat.getZona().getId() == loc.getZona().getId() && localitat.getFila() == loc.getFila() && localitat.getSeient() == loc.getSeient()) {
				return true;
			}
		}
		
		return false;
	}
	
	public String getFormattedTemps(Date data) {
		return new SimpleDateFormat("HH:mm").format(data);
	}
	
	public String getMessageCloseButton() {
		return messageCloseButton;
	}
	
	public Espectacle getEspectacle() {
		return espectacle;
	}
	
	public void setEspectacle(Espectacle espectacle) {
		this.espectacle = espectacle;
	}
	
	public Funcio getFuncio() {
		return funcio;
	}
	
	public void setFuncio(Funcio funcio) {
		this.funcio = funcio;
	}
	
	public Funcio getFuncioSel() {
		return funcioSel;
	}
	
	public void setFuncioSel(Funcio funcioSel) {
		this.funcioSel = funcioSel;
	}
	
	public Integer getIdRecinte() {
		return idRecinte;
	}
	
	public void setIdRecinte(Integer idRecinte) {
		this.idRecinte = idRecinte;
	}
	
	public Integer getIdEspectacle() {
		return idEspectacle;
	}
	
	public void setIdEspectacle(Integer idEspectacle) {
		this.idEspectacle = idEspectacle;
	}
	
	public Integer getIdFuncioModificar() {
		return idFuncioModificar;
	}
	
	public void setIdFuncioModificar(Integer idFuncioModificar) {
		this.idFuncioModificar = idFuncioModificar;
	}
	
	public Date getCurrentDate() {
		return currentDate;
	}
	
	public void setCurrentDate(Date currentDate) {
		this.currentDate = currentDate;
	}
	
	public List<Funcio> getFuncions() {
		return funcions;
	}
	
	public void setFuncions(List<Funcio> funcions) {
		this.funcions = funcions;
	}
	
	public List<SelectItem> getOpcionsEstatFuncio() {
		return opcionsEstatFuncio;
	}
	
	public void setOpcionsEstatFuncio(List<SelectItem> opcionsEstatFuncio) {
		this.opcionsEstatFuncio = opcionsEstatFuncio;
	}
	
	public List<SelectItem> getOpcionsEstatFuncioLlistat() {
		return opcionsEstatFuncioLlistat;
	}
	
	public void setOpcionsEstatFuncioLlistat(List<SelectItem> opcionsEstatFuncioLlistat) {
		this.opcionsEstatFuncioLlistat = opcionsEstatFuncioLlistat;
	}
	
	public List<Zona> getZones() {
		return zones;
	}
	
	public void setZones(List<Zona> zones) {
		this.zones = zones;
	}
	
}