package com.espectaclia.managed_beans;

import java.io.Serializable;

import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.SessionScoped;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;

import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.core.context.SecurityContextHolder;

import com.espectaclia.entities.Usuari;
import com.espectaclia.security.CustomUserDetails;

@ManagedBean(name="loginMB")
@SessionScoped
public class LoginMB implements Serializable {
  
	private static final long serialVersionUID = 1L;
	private static final String messageCloseButton = "<button type=\"button\" class=\"close\" onclick=\"$(this).parent().fadeOut()\"><span aria-hidden=\"true\">×</span></button>";
	
	private String userName = null; 
	private String password = null;
	private Authentication authentication = null;
	private Usuari loggedUser = null;
	private String tipusUser = null;
	
	@ManagedProperty(value="#{authenticationManager}")
	private AuthenticationManager authenticationManager = null;
	
	public String login() {
		try {
			Authentication request = new UsernamePasswordAuthenticationToken(this.getUserName(), this.getPassword());
			Authentication result = authenticationManager.authenticate(request);
			authentication = result;
			loggedUser = ((CustomUserDetails) authentication.getPrincipal()).getUsuari();
			tipusUser = authentication.getAuthorities().iterator().next().getAuthority();
			SecurityContextHolder.getContext().setAuthentication(result);
			
			return "/views/app/inici.xhtml?faces-redirect=true";
		} catch (AuthenticationException e) {
			e.printStackTrace();
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "L'usuari no existeix o la contrasenya és incorrecta." + messageCloseButton, "");
            FacesContext.getCurrentInstance().addMessage(null, message);
		}
		return null;
	}
	
	public String logout(){
		ExternalContext ec = FacesContext.getCurrentInstance().getExternalContext();
	    ec.invalidateSession();
	    
	    return "/views/public/login.xhtml?faces-redirect=true";
	}
	
	public String nouClient() {
		return "/views/public/nou_client.xhtml?faces-redirect=true";
	}
	
	public String getMessageCloseButton() {
		return messageCloseButton;
	}
	
	public String getUserName() {
		return userName;
	}
	
	public void setUserName(String userName) {
		this.userName = userName;
	}
	
	public String getPassword() {
		return password;
	}
	
	public void setPassword(String password) {
		this.password = password;
	}
	 
	public AuthenticationManager getAuthenticationManager() {
		return authenticationManager;
	}
	
	public void setAuthenticationManager(AuthenticationManager authenticationManager) {
		this.authenticationManager = authenticationManager;
	}
	
	public Authentication getAuthentication() {
		return authentication;
	}
	
	public void setAuthentication(Authentication authentication) {
		this.authentication = authentication;
	}
	
	public Usuari getLoggedUser() {
		return loggedUser;
	}
	
	public void setLoggedUser(Usuari loggedUser) {
		this.loggedUser = loggedUser;
	}
	
	public String getTipusUser() {
		return tipusUser;
	}
	
	public void setTipusUser(String tipusUser) {
		this.tipusUser = tipusUser;
	}
 
}

