package com.espectaclia.managed_beans;

import java.io.Serializable;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ViewScoped;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.servlet.ServletContext;

import com.espectaclia.entities.PreuLocalitatZona;
import com.espectaclia.entities.Espectacle;
import com.espectaclia.entities.PreuLocalitatZonaPK;
import com.espectaclia.entities.Zona;
import com.espectaclia.utils.RecinteRenderer;
import com.espectaclia.daos.PreuLocalitatZonaDAO;
import com.espectaclia.daos.EspectacleDAO;
import com.espectaclia.daos.ZonaDAO;

import org.primefaces.component.dialog.Dialog;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.context.support.WebApplicationContextUtils;

@ManagedBean(name="preusLocsMB")
@ViewScoped
public class PreusLocsMB implements Serializable{

	private static final long serialVersionUID = 1L;
	private static final String messageCloseButton = "<button type=\"button\" class=\"close\" onclick=\"$(this).parent().fadeOut()\"><span aria-hidden=\"true\">×</span></button>";
	
	@Autowired
	private PreuLocalitatZonaDAO preuLocalitatZonaDAO;
	
	@Autowired
	private EspectacleDAO espectacleDAO;
	
	@Autowired
	private ZonaDAO zonaDAO;
	
	@Autowired
	private RecinteRenderer recinteRenderer;

	private Espectacle espectacle;
	private PreuLocalitatZona preuLocalitatZona;
	private Zona zonaEditarPreu;
	private Integer idRecinte;
	private Integer idEspectacle;
	
	private Date currentDate = new Date();
	
	private List<PreuLocalitatZona> preusLocalitatZones;
	
	@PostConstruct
	public void init() {
		// Importem l'Autowired (Spring) al ManagedBean (JSF) per poder injectar els DAOs
		ExternalContext externalContext = FacesContext.getCurrentInstance().getExternalContext();
        ServletContext servletContext = (ServletContext) externalContext.getContext();
        WebApplicationContextUtils.getRequiredWebApplicationContext(servletContext).getAutowireCapableBeanFactory().autowireBean(this);
		
        espectacle = new Espectacle();
		preusLocalitatZones = new ArrayList<PreuLocalitatZona>();
    }
	
	public void loadEspectacle() {
		try {
			if (this.idEspectacle != null) {
				this.espectacle = espectacleDAO.cercarPerId(this.idEspectacle);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public void loadPreusLocalitatZones() {
		this.preusLocalitatZones = new ArrayList<PreuLocalitatZona>();
		
		try {
			this.preusLocalitatZones.addAll(preuLocalitatZonaDAO.cercarPerPropietat("espectacle", this.espectacle));
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public String getPreuZona(Zona zona) {
		for (PreuLocalitatZona preu : this.preusLocalitatZones) {
			if (preu.getZona().getId() == zona.getId()) {
				DecimalFormat format = new DecimalFormat("0.## €");
				return String.valueOf(format.format(preu.getPreu()));
			}
		}
		
		return "No definit";
	}
	
	public String editarPreuZona(Zona zonaEditarPreu) {
		this.zonaEditarPreu = zonaEditarPreu;
		
		boolean trobat = false;
		for (PreuLocalitatZona preu : this.preusLocalitatZones) {
			if (preu.getZona().getId() == zonaEditarPreu.getId()) {
				this.preuLocalitatZona = preu;
				trobat = true;
				break;
			}
		}
		
		if (!trobat) this.preuLocalitatZona = new PreuLocalitatZona();
		
		double zonaLeft = recinteRenderer.getZonaLeft(zonaEditarPreu, 600, 5);
		double zonaTop = recinteRenderer.getZonaTop(zonaEditarPreu, 500, 5);
		double zonaWidth = recinteRenderer.getZonaWidth(zonaEditarPreu, 600, 5);
		double zonaHeight = recinteRenderer.getZonaHeight(zonaEditarPreu, 500, 5);
		
		double dialogTop = zonaTop + 105 + (zonaHeight - 90)/2;
		double dialogLeft = zonaLeft + 10 + (zonaWidth - 240)/2;
		
		Dialog preuDialog = (Dialog) recinteRenderer.findComponent("zones_zona_dialog");
		preuDialog.setPosition(dialogLeft+","+dialogTop);

		return null;
	}
	
	public String tancarPreuZona() {
		this.zonaEditarPreu = null;
		this.preuLocalitatZona = null;
		
		return "/views/app/preuslocs/preuslocs.xhtml?faces-redirect=true&idRecinte="+idRecinte+"&idEspectacle="+idEspectacle;
	}
	
	public String confirmarPreuZona() {
		try {
			// Comprovem que la zona no tingui reserves pendents per a l'espectacle seleccionat
			if (this.zonaEditarPreu.teReservesPendents(this.espectacle.getId())) {
				// El preu per localitat de la zona no es pot modificar
				FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "El preu per localitat de la zona \""+this.zonaEditarPreu.getNom()+"\" per a l'espectacle \""+this.espectacle.getNom()+"\" no es pot modificar perquè té reserves pendents." + messageCloseButton, "");
				FacesContext.getCurrentInstance().addMessage(null, message);
				FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);
			} else {
				boolean trobat = false;
				boolean eliminarPreu = (this.preuLocalitatZona.getPreu() == null);
				for (PreuLocalitatZona preu : this.preusLocalitatZones) {
					if (preu.getZona().getId() == this.zonaEditarPreu.getId()) {
						if (!eliminarPreu) {
							preu.setPreu(this.preuLocalitatZona.getPreu());
						} else {
							for (PreuLocalitatZona preuObj : this.espectacle.getPreuLocalitatZones()) {
								if (preuObj.getPreuLocalitatZonaPK().equals(preu.getPreuLocalitatZonaPK())) {
									this.espectacle.getPreuLocalitatZones().remove(preuObj);
									break;
								}
							}
							
							for (PreuLocalitatZona preuObj : this.zonaEditarPreu.getPreuLocalitatZones()) {
								if (preuObj.getPreuLocalitatZonaPK().equals(preu.getPreuLocalitatZonaPK())) {
									this.zonaEditarPreu.getPreuLocalitatZones().remove(preuObj);
									break;
								}
							}
						}
						trobat = true;
						break;
					}
				}
				
				// Es crea un nou preu
				if (!trobat && !eliminarPreu) {
					this.preuLocalitatZona.setPreuLocalitatZonaPK(new PreuLocalitatZonaPK(this.zonaEditarPreu.getId(), this.espectacle.getId()));
					
					preuLocalitatZonaDAO.persistir(this.preuLocalitatZona);
					
					// El preu per localitat de la zona s'ha creat amb èxit
					FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_INFO, "El preu per localitat de la zona \""+this.zonaEditarPreu.getNom()+"\" per a l'espectacle \""+this.espectacle.getNom()+"\" s'ha creat amb èxit." + messageCloseButton, "");
					FacesContext.getCurrentInstance().addMessage(null, message);
					FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);
				// Es modifica un preu existent
				} else if (trobat && !eliminarPreu) {
					preuLocalitatZonaDAO.unir(this.preuLocalitatZona);
					
					// El preu per localitat de la zona s'ha modificat amb èxit
					FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_INFO, "El preu per localitat de la zona \""+this.zonaEditarPreu.getNom()+"\" per a l'espectacle \""+this.espectacle.getNom()+"\" s'ha modificat amb èxit." + messageCloseButton, "");
					FacesContext.getCurrentInstance().addMessage(null, message);
					FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);
				// S'elimina un preu existent
				} else if (trobat && eliminarPreu) {
					espectacleDAO.unir(this.espectacle);
					zonaDAO.unir(this.zonaEditarPreu);
					
					// El preu per localitat de la zona s'ha eliminat amb èxit
					FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_INFO, "El preu per localitat de la zona \""+this.zonaEditarPreu.getNom()+"\" per a l'espectacle \""+this.espectacle.getNom()+"\" s'ha eliminat amb èxit." + messageCloseButton, "");
					FacesContext.getCurrentInstance().addMessage(null, message);
					FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);
				}
			}
		} catch (Exception e) {
			// S'ha produït un error desconegut intentant modificar el preu per localitat de la zona
			e.printStackTrace();
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "No s'ha pogut modificar el preu per localitat de la zona \""+this.zonaEditarPreu.getNom()+"\" per a l'espectacle \""+this.espectacle.getNom()+"\". Si us plau, comprova que les dades siguin correctes." + messageCloseButton, "");
            FacesContext.getCurrentInstance().addMessage(null, message);
            return null;
		}
		
		return "/views/app/preuslocs/preuslocs.xhtml?faces-redirect=true&idRecinte="+idRecinte+"&idEspectacle="+idEspectacle;
	}
	
	public String tornarGestioEspectacles() {
		return "/views/app/espectacle/espectacles.xhtml?faces-redirect=true&idRecinte="+idRecinte;
	}
	
	public String getZonaStyle(Zona zona, int ampladaRecinte, int alturaRecinte, int margeLateral) {
		if (zona.getCoordInici() != null && zona.getCoordInici().trim().length() > 0 && zona.getCoordFi() != null && zona.getCoordFi().trim().length() > 0) {
			double left = recinteRenderer.getZonaLeft(zona, ampladaRecinte, margeLateral);
			double top = recinteRenderer.getZonaTop(zona, alturaRecinte, margeLateral);
			double width = recinteRenderer.getZonaWidth(zona, ampladaRecinte, margeLateral);
			double height = recinteRenderer.getZonaHeight(zona, alturaRecinte, margeLateral);
			
			return "left: "+left+"px;"+
				"top: "+top+"px;"+
				"width: "+width+"px;"+
				"height: "+height+"px;";
		} else {
			return "display: none";
		}
	}
	
	public List<Object> getBreadcrumbParams() {
		List<Object> params = new ArrayList<Object>();
		params.add(idRecinte);
		params.add(idEspectacle);
		
		return params;
	}
	
	public String getMessageCloseButton() {
		return messageCloseButton;
	}
	
	public Espectacle getEspectacle() {
		return espectacle;
	}
	
	public void setEspectacle(Espectacle espectacle) {
		this.espectacle = espectacle;
	}
	
	public PreuLocalitatZona getPreuLocalitatZona() {
		return preuLocalitatZona;
	}
	
	public void setPreuLocalitatZona(PreuLocalitatZona preuLocalitatZona) {
		this.preuLocalitatZona = preuLocalitatZona;
	}
	
	public Zona getZonaEditarPreu() {
		return zonaEditarPreu;
	}
	
	public void setZonaEditarPreu(Zona zonaEditarPreu) {
		this.zonaEditarPreu = zonaEditarPreu;
	}
	
	public Integer getIdRecinte() {
		return idRecinte;
	}
	
	public void setIdRecinte(Integer idRecinte) {
		this.idRecinte = idRecinte;
	}
	
	public Integer getIdEspectacle() {
		return idEspectacle;
	}
	
	public void setIdEspectacle(Integer idEspectacle) {
		this.idEspectacle = idEspectacle;
	}
	
	public Date getCurrentDate() {
		return currentDate;
	}
	
	public void setCurrentDate(Date currentDate) {
		this.currentDate = currentDate;
	}
	
	public List<PreuLocalitatZona> getPreusLocalitatZones() {
		return preusLocalitatZones;
	}
	
	public void setPreusLocalitatZones(List<PreuLocalitatZona> preusLocalitatZones) {
		this.preusLocalitatZones = preusLocalitatZones;
	}

}