package com.espectaclia.managed_beans;

import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.faces.application.ConfigurableNavigationHandler;
import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.ViewScoped;
import javax.faces.component.UIInput;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.faces.model.SelectItem;
import javax.servlet.ServletContext;

import org.primefaces.event.SelectEvent;

import com.espectaclia.daos.RecinteDAO;
import com.espectaclia.entities.Administrador;
import com.espectaclia.entities.Recinte;
import com.espectaclia.enumerations.TipusRecinte;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.context.support.WebApplicationContextUtils;

@ManagedBean(name="recinteMB")
@ViewScoped
public class RecinteMB implements Serializable{

	private static final long serialVersionUID = 1L;
	private static final String messageCloseButton = "<button type=\"button\" class=\"close\" onclick=\"$(this).parent().fadeOut()\"><span aria-hidden=\"true\">×</span></button>";
	
	@ManagedProperty(value="#{loginMB}")
    private LoginMB loginMB;
	
	@Autowired
	private RecinteDAO recinteDAO;

	private Recinte recinte;
	private Recinte recinteSel;
	private Integer idRecinteModificar;
	
	private List<Recinte> recintes;
	private List<SelectItem> opcionsTipusRecinte;
	private List<SelectItem> opcionsTipusRecinteLlistat;
	
	@PostConstruct
	public void init() {
		// Importem l'Autowired (Spring) al ManagedBean (JSF) per poder injectar els DAOs
		ExternalContext externalContext = FacesContext.getCurrentInstance().getExternalContext();
        ServletContext servletContext = (ServletContext) externalContext.getContext();
        WebApplicationContextUtils.getRequiredWebApplicationContext(servletContext).getAutowireCapableBeanFactory().autowireBean(this);
		
		recinte = new Recinte();
		recinteSel = null;
		recintes = new ArrayList<Recinte>();
		
		opcionsTipusRecinte = new ArrayList<SelectItem>();
	    opcionsTipusRecinte.add(new SelectItem(TipusRecinte.TEATRE, "Teatre"));
	    opcionsTipusRecinte.add(new SelectItem(TipusRecinte.CINEMA, "Cinema"));
	    opcionsTipusRecinte.add(new SelectItem(TipusRecinte.MUSEU, "Museu"));
	    opcionsTipusRecinte.add(new SelectItem(TipusRecinte.SALA_CONCERTS, "Sala de concerts"));
	    opcionsTipusRecinte.add(new SelectItem(TipusRecinte.SALA_EXPOSICIONS, "Sala d'exposicions"));
	    opcionsTipusRecinte.add(new SelectItem(TipusRecinte.SALA_POLIVALENT, "Sala polivalent"));
	    opcionsTipusRecinte.add(new SelectItem(TipusRecinte.SALO_CONGRESSOS, "Saló de congressos"));
	    opcionsTipusRecinte.add(new SelectItem(TipusRecinte.CENTRE_CULTURAL, "Centre cultural"));
	    opcionsTipusRecinte.add(new SelectItem(TipusRecinte.POLIESPORTIU, "Poliesportiu"));
	    opcionsTipusRecinte.add(new SelectItem(TipusRecinte.ESTADI, "Estadi"));
	    opcionsTipusRecinte.add(new SelectItem(TipusRecinte.FIRA, "Fira"));
	    opcionsTipusRecinte.add(new SelectItem(TipusRecinte.ALTRES, "Altres"));
		
	    opcionsTipusRecinteLlistat = new ArrayList<SelectItem>();
		opcionsTipusRecinteLlistat.add(new SelectItem("", "TOTS"));
		opcionsTipusRecinteLlistat.addAll(opcionsTipusRecinte);
    }
	
	public void loadRecintes() {
		this.recinte = new Recinte();
		this.recintes = new ArrayList<Recinte>();
		
		try {
			this.recintes.addAll(recinteDAO.cercarPerPropietat("administrador", loginMB.getLoggedUser()));
			
			// Si es tracta d'una modificació, busquem el recinte a modificar
			if (this.idRecinteModificar != null) {
				for (Recinte rec : this.recintes) {
					if (rec.getId() == this.idRecinteModificar) {
						this.recinte = rec;
						break;
					}
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public String crear() {
		try {
			// Comprovem si ja existeix un altre recinte amb el mateix nom del mateix administrador
			if (this.recintes.size() > 0) {
				for (Recinte recinteObj : this.recintes) {
					if (recinteObj.getNom().equalsIgnoreCase(recinte.getNom())) {
						UIInput uiInputUsuari = (UIInput) FacesContext.getCurrentInstance().getViewRoot().findComponent("recinte_form:recinte_nom_input");
						uiInputUsuari.setValid(false);
						
						FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "Ja tens un recinte amb el nom \""+recinte.getNom()+"\". Tria un altre nom." + messageCloseButton, "");
			            FacesContext.getCurrentInstance().addMessage(null, message);
			            return null;
					}
				}
			}
			
			recinte.setAdministrador((Administrador) loginMB.getLoggedUser());
			
			recinteDAO.persistir(this.recinte);

			// El nou recinte s'ha creat amb èxit
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_INFO, "El recinte s'ha creat amb èxit." + messageCloseButton, "");
			FacesContext.getCurrentInstance().addMessage(null, message);
			FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);
            return "/views/app/recinte/recintes.xhtml?faces-redirect=true";
		} catch (Exception e) {
			// S'ha produït un error desconegut intentant crear el nou recinte
			e.printStackTrace();
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "No s'ha pogut crear el recinte. Si us plau, comprova que les dades introduïdes siguin correctes." + messageCloseButton, "");
            FacesContext.getCurrentInstance().addMessage(null, message);
            return null;
		}
	}
	
	public String guardar() {
		try {
			// Comprovem si ja existeix un altre recinte amb el mateix nom del mateix administrador
			if (this.recintes.size() > 0) {
				for (Recinte recinteObj : this.recintes) {
					if (recinteObj.getNom().equalsIgnoreCase(recinte.getNom()) && recinteObj.getId() != recinte.getId()) {
						UIInput uiInputUsuari = (UIInput) FacesContext.getCurrentInstance().getViewRoot().findComponent("recinte_form:recinte_nom_input");
						uiInputUsuari.setValid(false);
						
						FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "Ja tens un recinte amb el nom \""+recinte.getNom()+"\". Tria un altre nom." + messageCloseButton, "");
			            FacesContext.getCurrentInstance().addMessage(null, message);
			            return null;
					}
				}
			}
			
			recinteDAO.unir(this.recinte);

			// El recinte s'ha modificat amb èxit
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_INFO, "El recinte s'ha modificat amb èxit." + messageCloseButton, "");
			FacesContext.getCurrentInstance().addMessage(null, message);
			FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);
			return "/views/app/recinte/recintes.xhtml?faces-redirect=true";
		} catch (Exception e) {
			// S'ha produït un error desconegut intentant modificar el recinte
			e.printStackTrace();
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "No s'ha pogut modificar el recinte. Si us plau, comprova que les dades modificades siguin correctes." + messageCloseButton, "");
            FacesContext.getCurrentInstance().addMessage(null, message);
            return null;
		}
	}
	
	public String eliminar(Recinte rec) {
		try {
			// Comprovem que el recinte no tingui cap espectacle
			if (rec.getEspectacles().size() > 0) {
				FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "El recinte no es pot eliminar perquè té espectacles associats. Elimina primer els espectacles." + messageCloseButton, "");
	            FacesContext.getCurrentInstance().addMessage(null, message);
	            return null;
			}
			
			recinteDAO.eliminar(rec.getId());
			
			// El recinte s'ha eliminat amb èxit
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_INFO, "El recinte s'ha eliminat amb èxit." + messageCloseButton, "");
			FacesContext.getCurrentInstance().addMessage(null, message);
			FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);
			return "/views/app/recinte/recintes.xhtml?faces-redirect=true";
		} catch (Exception e) {
			// S'ha produït un error desconegut intentant eliminar el recinte
			e.printStackTrace();
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "S'ha produït un error i no s'ha pogut eliminar el recinte." + messageCloseButton, "");
            FacesContext.getCurrentInstance().addMessage(null, message);
		}
		return null;
	}
	
	public void seleccionarRecinte(SelectEvent event) throws IOException {
		recinteSel = (Recinte) event.getObject();
		
		ConfigurableNavigationHandler configurableNavigationHandler = (ConfigurableNavigationHandler)FacesContext.getCurrentInstance().getApplication().getNavigationHandler();
		configurableNavigationHandler.performNavigation("/views/app/inici.xhtml?faces-redirect=true&idRecinte="+recinteSel.getId());
	}
	
	public String nouRecinte() {
		return "/views/app/recinte/modificar_recinte.xhtml?faces-redirect=true";
	}
	
	public String modificarRecinte(Integer id) {
		return "/views/app/recinte/modificar_recinte.xhtml?faces-redirect=true&idRecinteModificar="+id;
	}
	
	public String gestionarZones(Integer id) {
		return "/views/app/zona/zones.xhtml?faces-redirect=true&idRecinte="+id;
	}
	
	public String gestionarEspectacles(Integer id) {
		return "/views/app/espectacle/espectacles.xhtml?faces-redirect=true&idRecinte="+id;
	}
	
	public List<Object> getBreadcrumbParams() {
		List<Object> params = new ArrayList<Object>();
		params.add(idRecinteModificar);
		
		return params;
	}
	
	public String getMessageCloseButton() {
		return messageCloseButton;
	}
	
	public LoginMB getLoginMB() {
		return loginMB;
	}
	
	public void setLoginMB(LoginMB loginMB) {
		this.loginMB = loginMB;
	}
	
	public Recinte getRecinte() {
		return recinte;
	}
	
	public void setRecinte(Recinte recinte) {
		this.recinte = recinte;
	}
	
	public Recinte getRecinteSel() {
		return recinteSel;
	}
	
	public void setRecinteSel(Recinte recinteSel) {
		this.recinteSel = recinteSel;
	}
	
	public Integer getIdRecinteModificar() {
		return idRecinteModificar;
	}
	
	public void setIdRecinteModificar(Integer idRecinteModificar) {
		this.idRecinteModificar = idRecinteModificar;
	}
	
	public List<Recinte> getRecintes() {
		return recintes;
	}
	
	public void setRecintes(List<Recinte> recintes) {
		this.recintes = recintes;
	}
	
	public List<SelectItem> getOpcionsTipusRecinte() {
		return opcionsTipusRecinte;
	}
	
	public void setOpcionsTipusRecinte(List<SelectItem> opcionsTipusRecinte) {
		this.opcionsTipusRecinte = opcionsTipusRecinte;
	}
	
	public List<SelectItem> getOpcionsTipusRecinteLlistat() {
		return opcionsTipusRecinteLlistat;
	}
	
	public void setOpcionsTipusRecinteLlistat(List<SelectItem> opcionsTipusRecinteLlistat) {
		this.opcionsTipusRecinteLlistat = opcionsTipusRecinteLlistat;
	}

}