package com.espectaclia.managed_beans;

import java.io.IOException;
import java.io.Serializable;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.faces.application.ConfigurableNavigationHandler;
import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.ViewScoped;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.faces.model.SelectItem;
import javax.servlet.ServletContext;

import com.espectaclia.entities.Client;
import com.espectaclia.entities.Descompte;
import com.espectaclia.entities.Espectacle;
import com.espectaclia.entities.Funcio;
import com.espectaclia.entities.Localitat;
import com.espectaclia.entities.PreuLocalitatZona;
import com.espectaclia.entities.Reserva;
import com.espectaclia.entities.Taquiller;
import com.espectaclia.entities.Valoracio;
import com.espectaclia.entities.ValoracioPK;
import com.espectaclia.entities.Zona;
import com.espectaclia.enumerations.EstatFuncio;
import com.espectaclia.enumerations.EstatReserva;
import com.espectaclia.enumerations.Puntuacio;
import com.espectaclia.enumerations.TipusEspectacle;
import com.espectaclia.utils.RecinteRenderer;
import com.espectaclia.daos.ClientDAO;
import com.espectaclia.daos.EspectacleDAO;
import com.espectaclia.daos.LocalitatDAO;
import com.espectaclia.daos.ReservaDAO;
import com.espectaclia.daos.TaquillerDAO;
import com.espectaclia.daos.ValoracioDAO;
import com.espectaclia.daos.ZonaDAO;

import org.primefaces.component.dialog.Dialog;
import org.primefaces.event.SelectEvent;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.context.support.WebApplicationContextUtils;

@ManagedBean(name="reservaMB")
@ViewScoped
public class ReservaMB implements Serializable{

	private static final long serialVersionUID = 1L;
	private static final String messageCloseButton = "<button type=\"button\" class=\"close\" onclick=\"$(this).parent().fadeOut()\"><span aria-hidden=\"true\">×</span></button>";
	
	@ManagedProperty(value="#{loginMB}")
    private LoginMB loginMB;
	
	@Autowired
	private EspectacleDAO espectacleDAO;
	
	@Autowired
	private ReservaDAO reservaDAO;
	
	@Autowired
	private ZonaDAO zonaDAO;
	
	@Autowired
	private LocalitatDAO localitatDAO;
	
	@Autowired
	private ClientDAO clientDAO;
	
	@Autowired
	private TaquillerDAO taquillerDAO;
	
	@Autowired
	private ValoracioDAO valoracioDAO;
	
	@Autowired
	private RecinteRenderer recinteRenderer;

	private Reserva reserva;
	private Espectacle espectacle;
	private Funcio funcio;
	private Zona zona;
	private Client clientReserva;
	private Descompte descompte;
	private Valoracio valoracio;
	
	private String nomEspectacleCerca;
	private Integer idReservaModificar;
	private Integer idEspectacle;
	private Integer idFuncio;
	private Integer idZona;
	private Integer idClientReserva;
	
	private boolean editarNumLocsZonaNoNumerada;
	
	private boolean novaReserva;
	private boolean desdeInici;
	
	private List<Client> clients;
	
	private List<Reserva> reserves;
	private List<Espectacle> espectacles;
	private List<SelectItem> opcionsTipusEspectacleLlistat;
	private List<SelectItem> opcionsPuntuacioValoracio;
	
	private List<Zona> zones;
	private List<Localitat> localitatsSeleccionades;
	private List<Localitat> localitatsOcupades;
	private List<Localitat> localitatsBloquejades;
	
	@PostConstruct
	public void init() {
		// Importem l'Autowired (Spring) al ManagedBean (JSF) per poder injectar els DAOs
		ExternalContext externalContext = FacesContext.getCurrentInstance().getExternalContext();
        ServletContext servletContext = (ServletContext) externalContext.getContext();
        WebApplicationContextUtils.getRequiredWebApplicationContext(servletContext).getAutowireCapableBeanFactory().autowireBean(this);
		
        reserva = new Reserva();
		espectacle = new Espectacle();
		funcio = new Funcio();
		valoracio = new Valoracio();
		
		clients = new ArrayList<Client>();
		
		nomEspectacleCerca = "";
		reserves = new ArrayList<Reserva>();
		espectacles = new ArrayList<Espectacle>();
	    
	    opcionsTipusEspectacleLlistat = new ArrayList<SelectItem>();
		opcionsTipusEspectacleLlistat.add(new SelectItem("", "TOTS"));
		opcionsTipusEspectacleLlistat.add(new SelectItem(TipusEspectacle.OBRA_TEATRAL, "Obra teatral"));
		opcionsTipusEspectacleLlistat.add(new SelectItem(TipusEspectacle.DANSA, "Dansa"));
		opcionsTipusEspectacleLlistat.add(new SelectItem(TipusEspectacle.PELLICULA, "Pel·lícula"));
		opcionsTipusEspectacleLlistat.add(new SelectItem(TipusEspectacle.EXPOSICIO, "Exposició"));
		opcionsTipusEspectacleLlistat.add(new SelectItem(TipusEspectacle.CONCERT, "Concert"));
		opcionsTipusEspectacleLlistat.add(new SelectItem(TipusEspectacle.CONFERENCIA, "Conferència"));
		opcionsTipusEspectacleLlistat.add(new SelectItem(TipusEspectacle.ESDEVENIMENT_ESPORTIU, "Esdev. esportiu"));
		opcionsTipusEspectacleLlistat.add(new SelectItem(TipusEspectacle.CONGRES, "Congrés"));
	    opcionsTipusEspectacleLlistat.add(new SelectItem(TipusEspectacle.ALTRES, "Altres"));
	    
	    opcionsPuntuacioValoracio = new ArrayList<SelectItem>();
	    opcionsPuntuacioValoracio.add(new SelectItem(Puntuacio.EXCELLENT, "Excel·lent"));
	    opcionsPuntuacioValoracio.add(new SelectItem(Puntuacio.MOLT_BE, "Molt bé"));
	    opcionsPuntuacioValoracio.add(new SelectItem(Puntuacio.BE, "Bé"));
	    opcionsPuntuacioValoracio.add(new SelectItem(Puntuacio.REGULAR, "Regular"));
	    opcionsPuntuacioValoracio.add(new SelectItem(Puntuacio.MALAMENT, "Malament"));
    }
	
	public void loadReserves() {
		this.reserva = new Reserva();
		this.reserves = new ArrayList<Reserva>();
		
		try {
			if (loginMB.getTipusUser().equalsIgnoreCase("CLIENT")) { // Carguem les reserves del Client
				// Si el client té reserves temporals de més de 10 minuts, s'esborren
				Client client = (Client) loginMB.getLoggedUser();
				List<Reserva> reservesEsborrades = new ArrayList<Reserva>();
				for (Reserva reserva : client.getReserves()) {
					if (reserva.getEstat() == EstatReserva.TEMPORAL) {
						double minutsDiff = 0; 
						if (reserva.getDataUltModificacio() != null) {
							minutsDiff = ((new Date()).getTime() - reserva.getDataUltModificacio().getTime()) / 60000;
						} else {
							minutsDiff = ((new Date()).getTime() - reserva.getDataCreacio().getTime()) / 60000;
						}
						
						if (minutsDiff > 10) {
							reservesEsborrades.add(reserva);
							reservaDAO.eliminar(reserva);
						}
					}
				}
				List<Reserva> reserves = client.getReserves();
				reserves.removeAll(reservesEsborrades);
				
				List<Reserva> reservesNet = new ArrayList<Reserva>();
				reservesNet.addAll(reserves);
				
				client.setReserves(reservesNet);
				client = clientDAO.unir(client);
				loginMB.setLoggedUser(client);
				
				this.reserves.addAll(reservaDAO.cercarReservesClient((Client) loginMB.getLoggedUser()));
			} else if (loginMB.getTipusUser().equalsIgnoreCase("TAQUILLER")) { // Carguem les reserves del Recinte associat al Taquiller
				// Si el recinte del taquiller té reserves temporals de més de 10 minuts, s'esborren
				Taquiller taquiller = (Taquiller) loginMB.getLoggedUser();
				
				for (Zona zona : taquiller.getRecinte().getZones()) {
					for (Reserva reserva : zona.getReserves()) {
						if (reserva.getEstat() == EstatReserva.TEMPORAL) {
							double minutsDiff = 0; 
							if (reserva.getDataUltModificacio() != null) {
								minutsDiff = ((new Date()).getTime() - reserva.getDataUltModificacio().getTime()) / 60000;
							} else {
								minutsDiff = ((new Date()).getTime() - reserva.getDataCreacio().getTime()) / 60000;
							}
							
							if (minutsDiff > 10) reservaDAO.eliminar(reserva);
						}
					}
				}
				
				taquiller = taquillerDAO.cercarPerId(((Taquiller) loginMB.getLoggedUser()).getId());
				loginMB.setLoggedUser(taquiller);
				
				this.reserves.addAll(reservaDAO.cercarReservesTaquiller((Taquiller) loginMB.getLoggedUser()));
			}
			
			// Si es tracta d'una modificació, busquem la reserva a modificar
			if (this.idReservaModificar != null) {
				this.reserva = reservaDAO.cercarPerId(this.idReservaModificar);
				if (this.reserva.getId() != null) {
					this.espectacle = this.reserva.getFuncio().getEspectacle();
					this.funcio = this.reserva.getFuncio();
					this.zona = this.reserva.getZona();
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public void loadEspectacles() {
		this.espectacles = new ArrayList<Espectacle>();
		
		try {
			if (this.nomEspectacleCerca != null && this.nomEspectacleCerca.trim().length() > 0) {
				this.espectacles.addAll(espectacleDAO.cercarPerNom(this.nomEspectacleCerca));
			} else {
				this.espectacles.addAll(espectacleDAO.cercarTots());
			}
			
			if (this.idEspectacle != null) {
				for (Espectacle esp : this.espectacles) {
					if (esp.getId() == this.idEspectacle) {
						this.espectacle = esp;
						
						if (this.idFuncio != null) {
							for (Funcio fun : espectacle.getFuncions()) {
								if (fun.getId() == this.idFuncio) {
									this.funcio = fun;
								}
							}
						}
						
						break;
					}
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public void loadZones() {
		this.zones = new ArrayList<Zona>();
		
		try {
			this.zones.addAll(zonaDAO.cercarPerPropietat("recinte", this.espectacle.getRecinte()));
			
			// Generem les localitats per pintar el pati de butaques de les zones amb localitats numerades
			for (Zona zon : this.zones) {
				if (this.idZona != null) {
					if (zon.getId() == this.idZona) {
						this.zona = zon;
					}
				}
				
				if (zon.getNumFiles() != null && zon.getNumSeientsFila() != null) {
					List<Localitat> localitats = new ArrayList<Localitat>();
					
					for (int numFila=1; numFila<=zon.getNumFiles(); numFila++) {
						for (int numSeient=1; numSeient<=zon.getNumSeientsFila(); numSeient++) {
							Localitat loc = new Localitat();
							loc.setFila(numFila);
							loc.setSeient(numSeient);
							loc.setZona(zon);
							
							localitats.add(loc);
						}
					}
					
					zon.setLocalitatsPatiButaques(localitats);
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public void loadEstatsLocalitats() {
		this.localitatsSeleccionades = new ArrayList<Localitat>();
		this.localitatsOcupades = new ArrayList<Localitat>();
		this.localitatsBloquejades = new ArrayList<Localitat>();
		
		if (this.funcio != null) {
			for (Reserva reservaObj : this.funcio.getReserves()) {
				if (reservaObj.getEstat() == EstatReserva.BLOQUEJADA) { // Són localitats bloquejades per Administrador
					for (Localitat localitat : reservaObj.getLocalitats()) {
						this.localitatsBloquejades.add(localitat);
					}
				} else if (reservaObj.getId() == this.reserva.getId()) { // Són localitats de la reserva actual
					for (Localitat localitat : reservaObj.getLocalitats()) {
						this.localitatsSeleccionades.add(localitat);
					}
				} else if (reservaObj.getEstat() == EstatReserva.CONFIRMADA ||
						reservaObj.getEstat() == EstatReserva.VENUDA ||
						reservaObj.getEstat() == EstatReserva.TEMPORAL) { // Són localitats reservades per un altre
					
					for (Localitat localitat : reservaObj.getLocalitats()) {
						this.localitatsOcupades.add(localitat);
					}
				}
					
			}
		}
	}
	
	public void loadClients() {
		this.clients = new ArrayList<Client>();
		
		try {
			this.clients.addAll(clientDAO.cercarTots());
			
			// Eliminem el client especial per bloquejar localitats
			for (Client client : this.clients) {
				if (client.getId() == 9999) {
					this.clients.remove(client);
					break;
				}
			}
			
			// Si hi s'ha editat el client de la reserva, el busquem
			for (Client client : this.clients) {
				if (client.getId() == this.idClientReserva) {
					this.clientReserva = client;
					break;
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public List<Client> autocompleteClients(String query) {
        List<Client> clientsFiltrats = new ArrayList<Client>();
         
        for (Client client : clients) {
            if ((client.getCognoms()+" "+client.getNom()).toLowerCase().contains(query.toLowerCase())) {
                clientsFiltrats.add(client);
            }
        }
         
        return clientsFiltrats;
    }
	
	public String getFormattedNomClient(Client client) {
		if (client != null && client.getNom() != null && client.getCognoms() != null) {
			return client.getCognoms()+", "+client.getNom();
		}
		return "";
	}
	
	public String confirmarReserva() {
		try {
			// La reserva ha de tenir com a mínim una localitat seleccionada
			if (this.reserva.getNumLocalitats() > 0) {
				if (loginMB.getTipusUser() == "CLIENT") {
					Client client = (Client) loginMB.getLoggedUser();
					for (Reserva reserva : client.getReserves()) {
						if (reserva.getId() == this.reserva.getId()) {
							long codiQR = (this.reserva.getId() * this.reserva.getZona().getId() * this.reserva.getFuncio().getId()) ^ this.reserva.getClient().getId();
							
							reserva.setCodiQR(String.valueOf(codiQR));
							reserva.setPreuFinal(calcularPreuReserva());
							reserva.setEstat(EstatReserva.CONFIRMADA);
							reserva.setDataUltModificacio(new Date());
							
							client = clientDAO.unir(client);
							loginMB.setLoggedUser(client);
							
							break;
						}
					}
				} else if (loginMB.getTipusUser() == "TAQUILLER") {
					long codiQR = (this.reserva.getId() * this.reserva.getZona().getId() * this.reserva.getFuncio().getId()) ^ this.reserva.getClient().getId();
					
					this.reserva.setCodiQR(String.valueOf(codiQR));
					this.reserva.setPreuFinal(calcularPreuReserva());
					this.reserva.setEstat(EstatReserva.CONFIRMADA);
					this.reserva.setDataUltModificacio(new Date());
					
					reservaDAO.unir(this.reserva);
				}

				// La reserva s'ha creat/modificat amb èxit
				return confirmacio();
			} else {
				FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "Has de sel·leccionar com a mínim una localitat per reservar." + messageCloseButton, "");
	            FacesContext.getCurrentInstance().addMessage(null, message);
	            return null;
			}
		} catch (Exception e) {
			// S'ha produït un error desconegut intentant crear/modificar la reserva
			e.printStackTrace();
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "No s'ha pogut "+(this.novaReserva ? "crear" : "modificar")+" la reserva. Si us plau, comprova que les dades siguin correctes." + messageCloseButton, "");
            FacesContext.getCurrentInstance().addMessage(null, message);
            return null;
		}
	}
	
	public String confirmarVendaReserva() {
		try {
			if (loginMB.getTipusUser() == "CLIENT") {
				Client client = (Client) loginMB.getLoggedUser();
				for (Reserva reserva : client.getReserves()) {
					if (reserva.getId() == this.reserva.getId()) {
						reserva.setPreuFinal(calcularPreuFinalReserva());
						reserva.setEstat(EstatReserva.VENUDA);
						reserva.setDataUltModificacio(new Date());
						
						client = clientDAO.unir(client);
						loginMB.setLoggedUser(client);
						
						break;
					}
				}
			} else if (loginMB.getTipusUser() == "TAQUILLER") {
				this.reserva.setPreuFinal(calcularPreuFinalReserva());
				this.reserva.setEstat(EstatReserva.VENUDA);
				this.reserva.setDataUltModificacio(new Date());
				
				reservaDAO.unir(this.reserva);
			}

			// La reserva s'ha venut amb èxit
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_INFO, "La reserva s'ha venut amb èxit." + messageCloseButton, "");
			FacesContext.getCurrentInstance().addMessage(null, message);
			FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);
			return tornarGestioReserves();
		} catch (Exception e) {
			// S'ha produït un error desconegut intentant vendre la reserva
			e.printStackTrace();
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "No s'ha pogut vendre la reserva. Si us plau, comprova que les dades siguin correctes." + messageCloseButton, "");
            FacesContext.getCurrentInstance().addMessage(null, message);
            return null;
		}
	}
	
	public String confirmarValoracioEspectacle() {
		try {
			this.valoracio.setValoracioPK(new ValoracioPK(this.reserva.getClient().getId(), this.espectacle.getId()));
			
			valoracioDAO.persistir(this.valoracio);

			// La valoració s'ha creat amb èxit
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_INFO, "La teva valoració de l'espectacle \""+this.espectacle.getNom()+"\" s'ha enregistrat amb èxit. Moltes gràcies per compartir la teva opinió." + messageCloseButton, "");
			FacesContext.getCurrentInstance().addMessage(null, message);
			FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);
            return tornarGestioReserves();
		} catch (Exception e) {
			// S'ha produït un error desconegut intentant crear la valoració
			e.printStackTrace();
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "No s'ha pogut enregistrar la valoració. Si us plau, comprova que les dades introduïdes siguin correctes." + messageCloseButton, "");
            FacesContext.getCurrentInstance().addMessage(null, message);
            return null;
		}
	}
	
	public String cancelar(Reserva res) {
		try {
			// Comprovem que la reserva tingui estat CONFIRMADA
			if (res.getEstat() != EstatReserva.CONFIRMADA) {
				FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "Només es poden cancel·lar reserves confirmades." + messageCloseButton, "");
	            FacesContext.getCurrentInstance().addMessage(null, message);
	            return null;
			}
			
			if (loginMB.getTipusUser() == "CLIENT") {
				Client client = (Client) loginMB.getLoggedUser();
				for (Reserva reserva : client.getReserves()) {
					if (reserva.getId() == res.getId()) {
						reserva.setEstat(EstatReserva.CANCELLADA);
						reserva.setDataUltModificacio(new Date());
						
						client = clientDAO.unir(client);
						loginMB.setLoggedUser(client);
						
						break;
					}
				}
			} else if (loginMB.getTipusUser() == "TAQUILLER") {
				res.setEstat(EstatReserva.CANCELLADA);
				res.setDataUltModificacio(new Date());
				
				res = reservaDAO.unir(res);
			}
			
			// La reserva s'ha cancel·lat amb èxit
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_INFO, "La reserva s'ha cancel·lat amb èxit." + messageCloseButton, "");
			FacesContext.getCurrentInstance().addMessage(null, message);
			FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);
			return tornarGestioReserves();
		} catch (Exception e) {
			// S'ha produït un error desconegut intentant cancel·lar la reserva
			e.printStackTrace();
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "S'ha produït un error i no s'ha pogut cancel·lar la reserva." + messageCloseButton, "");
            FacesContext.getCurrentInstance().addMessage(null, message);
		}
		return null;
	}
	
	public List<Zona> getZonesLocNumerades() {
		List<Zona> zonesLocNumerades = new ArrayList<Zona>();
		for (Zona zona : this.zones) {
			if (zona.getNumFiles() != null && zona.getNumSeientsFila() != null) {
				zonesLocNumerades.add(zona);
			}
		}
		
		return zonesLocNumerades;
	}
	
	public List<Zona> getZonesLocNoNumerades() {
		List<Zona> zonesLocNoNumerades = new ArrayList<Zona>();
		for (Zona zona : this.zones) {
			if (zona.getNumFiles() == null && zona.getNumSeientsFila() == null) {
				zonesLocNoNumerades.add(zona);
			}
		}
		
		return zonesLocNoNumerades;
	}
	
	public void seleccionarEspectacle(SelectEvent event) throws IOException {
		Espectacle espectacleSel = (Espectacle) event.getObject();
		
		ConfigurableNavigationHandler configurableNavigationHandler = (ConfigurableNavigationHandler) FacesContext.getCurrentInstance().getApplication().getNavigationHandler();
		configurableNavigationHandler.performNavigation("/views/app/reserva/espectacle.xhtml?faces-redirect=true&novaReserva=true&desdeInici="+this.desdeInici+"&idEspectacle="+espectacleSel.getId());
	}
	
	public void seleccionarClient(SelectEvent event) throws IOException {
		this.clientReserva = (Client) event.getObject();
	}
	
	public void seleccionarFuncio(SelectEvent event) throws IOException {
		Funcio funcioSel = (Funcio) event.getObject();
		// Comprovem que si l'usuari es un Taquiller, hagi triat un Client
		if (loginMB.getTipusUser() == "TAQUILLER" && this.novaReserva && this.clientReserva == null) {
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "Si us plau, cerca un client sobre el què fer la reserva." + messageCloseButton, "");
            FacesContext.getCurrentInstance().addMessage(null, message);
            return;
		}
		
		// Comprovem que no sigui una funció amb estat CANCELLADA
		if (funcioSel.getEstat() == EstatFuncio.CANCELLADA) {
			event.getComponent();
			
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "Aquesta funció ha estat cancel·lada i no s'hi poden fer reserves de localitats." + messageCloseButton, "");
            FacesContext.getCurrentInstance().addMessage(null, message);
			return;
		}
		
		// Si l'usuari és un Client, comprovem que aquest no tingui cap altra reserva per a la funció sel·leccionada
		if (loginMB.getTipusUser() == "CLIENT") {
			for (Reserva reserva : ((Client) loginMB.getLoggedUser()).getReserves()) {
				if (reserva.getFuncio().getId() == funcioSel.getId() && (reserva.getEstat() == EstatReserva.CONFIRMADA || reserva.getEstat() == EstatReserva.VENUDA) && (this.idReservaModificar == null || this.idReservaModificar != reserva.getId())) {
					FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "Ja tens una reserva cursada per a aquesta funció." + messageCloseButton, "");
		            FacesContext.getCurrentInstance().addMessage(null, message);
					return;
				}
			}
		// Si l'usuari és un Taquiller, comprovem que l'usuari de la reserva no tingui cap altra reserva per a la funció sel·leccionada
		} else if (loginMB.getTipusUser() == "TAQUILLER") {
			for (Reserva reserva : (this.novaReserva ? this.clientReserva.getReserves() : this.reserva.getClient().getReserves())) {
				if (reserva.getFuncio().getId() == funcioSel.getId() && (reserva.getEstat() == EstatReserva.CONFIRMADA || reserva.getEstat() == EstatReserva.VENUDA) && (this.idReservaModificar == null || this.idReservaModificar != reserva.getId())) {
					FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "El client "+this.clientReserva.getNom()+" "+this.clientReserva.getCognoms()+" ja té una reserva cursada per a aquesta funció." + messageCloseButton, "");
		            FacesContext.getCurrentInstance().addMessage(null, message);
					return;
				}
			}
		}
		
		// Si es tracta d'una modificació de reserva i l'usuari escull una altra funció, cal eliminar les localitats de la reserva
		if (this.idReservaModificar != null && this.funcio.getId() != funcioSel.getId()) {
			try {
				for (Localitat locReserva : this.reserva.getLocalitats()) {
					localitatDAO.eliminar(locReserva);
				}
				
				List<Localitat> localitatsNet = new ArrayList<Localitat>();
				
				this.reserva.setLocalitats(localitatsNet);
				this.reserva.setNumLocalitats(0);
				this.reserva.setEstat(EstatReserva.TEMPORAL);
				this.reserva.setDataUltModificacio(new Date());
				
				this.reserva = reservaDAO.unir(this.reserva);
				
				if (loginMB.getTipusUser().equalsIgnoreCase("CLIENT")) {
					Client client = clientDAO.cercarPerId(((Client) loginMB.getLoggedUser()).getId());
					loginMB.setLoggedUser(client);
				} else if (loginMB.getTipusUser().equalsIgnoreCase("TAQUILLER")) {
					Taquiller taquiller = taquillerDAO.cercarPerId(((Taquiller) loginMB.getLoggedUser()).getId());
					loginMB.setLoggedUser(taquiller);
				}
			} catch (Exception e) {
				e.printStackTrace();
				return;
			}
		}
		
		String url = "/views/app/reserva/localitats.xhtml?faces-redirect=true"+
				"&novaReserva="+this.novaReserva+
				"&desdeInici="+this.desdeInici+
				"&idEspectacle="+this.espectacle.getId()+
				"&idFuncio="+funcioSel.getId();
		
		if (this.reserva.getZona() != null) url+= "&idZona="+this.reserva.getZona().getId();
		if (this.idReservaModificar != null) url += "&idReservaModificar="+this.idReservaModificar;
		if (this.clientReserva != null) url += "&idClientReserva="+this.clientReserva.getId();
		
		ConfigurableNavigationHandler configurableNavigationHandler = (ConfigurableNavigationHandler) FacesContext.getCurrentInstance().getApplication().getNavigationHandler();
		configurableNavigationHandler.performNavigation(url);
	}
	
	public String cercarDesdeInici() {
		return "/views/app/reserva/cercar_espectacles.xhtml?faces-redirect=true&includeViewParams=true&novaReserva=true&desdeInici=true";
	}
	
	public String tornarDesdeEspectacle() {
		if (novaReserva) {
			return "/views/app/reserva/cercar_espectacles.xhtml?faces-redirect=true&novaReserva=true&desdeInici="+this.desdeInici;
		} else {
			return tornarGestioReserves();
		}
	}
	
	public String tornarDesdeLocalitats() {
		return "/views/app/reserva/espectacle.xhtml?faces-redirect=true"+
				"&novaReserva="+this.novaReserva+
				"&desdeInici="+this.desdeInici+
				(this.idReservaModificar != null ? "&idReservaModificar="+this.idReservaModificar : "&idEspectacle="+this.idEspectacle);
	}
	
	public String tornarGestioReserves() {
		return "/views/app/reserva/reserves.xhtml?faces-redirect=true";
	}
	
	public String novaReserva() {
		return "/views/app/reserva/cercar_espectacles.xhtml?faces-redirect=true&novaReserva=true&desdeInici=false";
	}
	
	public String modificarReserva(Integer id) {
		return "/views/app/reserva/espectacle.xhtml?faces-redirect=true&novaReserva=false&desdeInici=false&idReservaModificar="+id;
	}
	
	public String vendreReserva(Integer id) {
		return "/views/app/reserva/vendre_reserva.xhtml?faces-redirect=true&idReservaModificar="+id;
	}
	
	public String valorarEspectacle(Reserva reserva) {
		// Comprovem que el client no hagi valorat prèviament aquest espectacle
		for (Valoracio valoracio : reserva.getFuncio().getEspectacle().getValoracions()) {
			if (valoracio.getClient().getId() == loginMB.getLoggedUser().getId()) {
				FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "Ja has valorat anteriorment aquest espectacle. Només es permet una valoració per usuari." + messageCloseButton, "");
	            FacesContext.getCurrentInstance().addMessage(null, message);
				return null;
			}
		}
		
		return "/views/app/reserva/valorar_espectacle.xhtml?faces-redirect=true&idReservaModificar="+reserva.getId();
	}
	
	public String confirmacio() {
		return "/views/app/reserva/confirmacio.xhtml?faces-redirect=true&novaReserva="+this.novaReserva+"&idReservaModificar="+this.idReservaModificar;
	}
	
	public String tancarNumLocalitatsZona() {
		this.editarNumLocsZonaNoNumerada = false;
		
		String url = "/views/app/reserva/localitats.xhtml?faces-redirect=true"+
				"&novaReserva="+this.novaReserva+
				"&desdeInici="+this.desdeInici+
				"&idEspectacle="+this.espectacle.getId()+
				"&idFuncio="+this.funcio.getId()+
				"&idZona="+this.zona.getId();
		
		if (this.idReservaModificar != null) url+="&idReservaModificar="+this.reserva.getId();
		if (this.idClientReserva != null) url += "&idClientReserva="+this.idClientReserva;
		
		return url;
	}
	
	public String confirmarNumLocalitatsZona() {
		try {
			long numLocalitatsZona = this.reserva.getNumLocalitats();
			// Comprovem que el número de localitats sigui superior a 0 i inferior o igual al nº de localitats disponibles de la zona
			if (numLocalitatsZona == 0 || numLocalitatsZona > getNumLocalitatsDisponibles(this.zona)) {
				FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "El nº de localitats ha de ser major a 0 i inferior o igual al nº de localitats disponibles de la zona." + messageCloseButton, "");
				FacesContext.getCurrentInstance().addMessage(null, message);
				FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);
				
				return null;
			} else {
				this.editarNumLocsZonaNoNumerada = false;
				
				// Editem la reserva si ja existeix i la possem en estat TEMPORAL
				if (this.reserva.getId() != null) {
					this.reserva.setZona(this.zona);
					this.reserva.setEstat(EstatReserva.TEMPORAL);
					this.reserva.setDataUltModificacio(new Date());
					
					this.reserva = reservaDAO.unir(this.reserva);
					
					if (loginMB.getTipusUser().equalsIgnoreCase("CLIENT")) {
						Client client = clientDAO.cercarPerId(((Client) loginMB.getLoggedUser()).getId());
						loginMB.setLoggedUser(client);
					} else if (loginMB.getTipusUser().equalsIgnoreCase("TAQUILLER")) {
					}
				// Cal crear una nova reserva TEMPORAL
				} else {
					Client client = new Client();
					if (loginMB.getTipusUser().equalsIgnoreCase("CLIENT")) {
						client = (Client) loginMB.getLoggedUser();
					} else if (loginMB.getTipusUser().equalsIgnoreCase("TAQUILLER")) {
						client = this.clientReserva;
					}
					this.reserva.setFuncio(this.funcio);
					this.reserva.setZona(this.zona);
					this.reserva.setClient(client);
					this.reserva.setCodiQR("");
					this.reserva.setPreuFinal(0);
					this.reserva.setEstat(EstatReserva.TEMPORAL);
					this.reserva.setDataCreacio(new Date());
					
					this.reserva = reservaDAO.persistir(this.reserva);
					
					client.getReserves().add(this.reserva);
					if (loginMB.getTipusUser().equalsIgnoreCase("CLIENT")) {
						client = clientDAO.cercarPerId(client.getId());
						loginMB.setLoggedUser(client);
					} else if (loginMB.getTipusUser().equalsIgnoreCase("TAQUILLER")) {
						Taquiller taquiller = taquillerDAO.cercarPerId(((Taquiller) loginMB.getLoggedUser()).getId());
						loginMB.setLoggedUser(taquiller);
					}
					
					this.idReservaModificar = this.reserva.getId();
				}
			}
		} catch (Exception e) {
			// S'ha produït un error desconegut intentant actualitzar la reserva
			e.printStackTrace();
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "No s'ha pogut modificar el nº de localitats a reservar. Si us plau, comprova que les dades siguin correctes." + messageCloseButton, "");
            FacesContext.getCurrentInstance().addMessage(null, message);
            return null;
		}
		
		String url = "/views/app/reserva/localitats.xhtml?faces-redirect=true"+
				"&novaReserva="+this.novaReserva+
				"&desdeInici="+this.desdeInici+
				"&idReservaModificar="+this.reserva.getId()+
				"&idEspectacle="+this.espectacle.getId()+
				"&idFuncio="+this.funcio.getId()+
				"&idZona="+this.zona.getId();
		
		if (this.idClientReserva != null) url += "&idClientReserva="+this.idClientReserva;
		
		return url;
	}
	
	public List<Object> getBreadcrumbParams() {
		List<Object> params = new ArrayList<Object>();
		params.add(novaReserva);
		params.add(desdeInici);
		params.add(idReservaModificar);
		params.add(idEspectacle);
		params.add(idFuncio);
		params.add(idZona);
		params.add(idClientReserva);
		
		return params;
	}
	
	public List<Descompte> getDescomptesAplicablesFuncio() {
		List<Descompte> descomptes = new ArrayList<Descompte>();
		
		for (Descompte descompte : this.espectacle.getDescomptes()) {
			boolean aplicable = true;
			if (descompte.getDataInici() != null && descompte.getDataInici().before(this.funcio.getData())) aplicable = false;
			if (descompte.getDataFi() != null && descompte.getDataFi().after(this.funcio.getData())) aplicable = false;
				
			if (aplicable) descomptes.add(descompte);
		}
		
		return descomptes;
	}
	
	public long getNumLocalitatsDisponibles(Zona zon) {
		long numLocs = zon.getAforament();
		
		for (Reserva reserva : zon.getReserves()) {
			if (reserva.getFuncio().getId() == this.funcio.getId()) {
				numLocs -= reserva.getNumLocalitats();
			}
		}
		
		return numLocs;
	}
	
	public String getPreuZona(Zona zon) {
		for (PreuLocalitatZona preu : this.espectacle.getPreuLocalitatZones()) {
			if (preu.getZona().getId() == zon.getId()) {
				DecimalFormat format = new DecimalFormat("0.## €");
				return String.valueOf(format.format(preu.getPreu()));
			}
		}
		
		return "No definit";
	}
	
	public String getPreuLocalitat(Localitat loc) {
		for (PreuLocalitatZona preu : this.espectacle.getPreuLocalitatZones()) {
			if (preu.getZona().getId() == loc.getZona().getId()) {
				DecimalFormat format = new DecimalFormat("0.## €");
				return String.valueOf(format.format(preu.getPreu()));
			}
		}
		
		return "No definit";
	}
	
	public double calcularPreuReserva() {
		if (this.reserva != null) {
			for (PreuLocalitatZona preu : this.espectacle.getPreuLocalitatZones()) {
				if (preu.getZona().getId() == this.reserva.getZona().getId()) {
					return (this.reserva.getNumLocalitats() * preu.getPreu());
				}
			}
		}
		return 0;
	}
	
	public String getPreuReserva() {
		DecimalFormat format = new DecimalFormat("0.## €");
		return String.valueOf(format.format(calcularPreuReserva()));
	}
	
	public double calcularPreuFinalReserva() {
		double preuFinal = calcularPreuReserva();
		if (this.descompte != null) preuFinal = preuFinal - (preuFinal * this.descompte.getPercDescompte() / 100);
		
		return preuFinal;
	}
	
	public String getPreuFinalReserva() {
		DecimalFormat format = new DecimalFormat("0.## €");
		return String.valueOf(format.format(calcularPreuFinalReserva()));
	}
	
	public String getPreuReservaDescompte() {
		String text = "";
		if (this.descompte != null) {
			text = getPreuReserva()+" - "+this.descompte.getPercDescompte()+"% = ";
		}
		
		return text;
	}
	
	public String getZonaStyle(Zona zona, int ampladaRecinte, int alturaRecinte, int margeLateral) {
		if (zona.getCoordInici() != null && zona.getCoordInici().trim().length() > 0 && zona.getCoordFi() != null && zona.getCoordFi().trim().length() > 0) {
			double left = recinteRenderer.getZonaLeft(zona, ampladaRecinte, margeLateral);
			double top = recinteRenderer.getZonaTop(zona, alturaRecinte, margeLateral);
			double width = recinteRenderer.getZonaWidth(zona, ampladaRecinte, margeLateral);
			double height = recinteRenderer.getZonaHeight(zona, alturaRecinte, margeLateral);
			
			return "left: "+left+"px;"+
				"top: "+top+"px;"+
				"width: "+width+"px;"+
				"height: "+height+"px;";
		} else {
			return "display: none";
		}
	}
	
	public String getLocalitatZonaStyle(Localitat localitat, Zona zona, int ampladaRecinte, int alturaRecinte, int margeLateralZona, int margeLateralLocalitat) {
		double left = recinteRenderer.getLocalitatZonaLeft(localitat, zona, ampladaRecinte, margeLateralZona, margeLateralLocalitat);
		double top = recinteRenderer.getLocalitatZonaTop(localitat, zona, alturaRecinte, margeLateralZona, margeLateralLocalitat);
		double width = recinteRenderer.getLocalitatZonaWidth(localitat, zona, ampladaRecinte, margeLateralZona, margeLateralLocalitat);
		double height = recinteRenderer.getLocalitatZonaHeight(localitat, zona, alturaRecinte, margeLateralZona, margeLateralLocalitat);
		
		return "left: "+left+"px;"+
				"top: "+top+"px;"+
				"width: "+width+"px;"+
				"height: "+height+"px;";
	}
	
	public String seleccionarZonaNoNumerada(Zona zon) {
		try {
			// Si la zona no té preu assignat, avisem
			if (getPreuZona(zon) == "No definit") {
				FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "No es poden reservar localitats de zones sense un preu per localitat definit." + messageCloseButton, "");
	            FacesContext.getCurrentInstance().addMessage(null, message);
	            return null;
			}
			// Si la zona és diferent a la seleccionada...
			if (this.zona != null && this.zona.getId() != zon.getId()) {
				// Si hi ha localitats, avisem
				if (this.reserva != null && this.reserva.getLocalitats().size() > 0) {
					FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "No es poden reservar localitats de diferents zones." + messageCloseButton, "");
		            FacesContext.getCurrentInstance().addMessage(null, message);
		            return null;
				}
			}
			
			this.zona = zon;
			this.editarNumLocsZonaNoNumerada = true;
			
			// Si estem modificant una reserva, actualitzem la zona
			if (this.reserva.getId() != null) {
				this.reserva.setZona(this.zona);
				reservaDAO.unir(this.reserva);
				
				if (loginMB.getTipusUser().equalsIgnoreCase("CLIENT")) {
					Client client = clientDAO.cercarPerId(((Client) loginMB.getLoggedUser()).getId());
					loginMB.setLoggedUser(client);
				} else if (loginMB.getTipusUser().equalsIgnoreCase("TAQUILLER")) {
					Taquiller taquiller = taquillerDAO.cercarPerId(((Taquiller) loginMB.getLoggedUser()).getId());
					loginMB.setLoggedUser(taquiller);
				}
			}
			
			double zonaLeft = recinteRenderer.getZonaLeft(this.zona, 600, 5);
			double zonaTop = recinteRenderer.getZonaTop(this.zona, 500, 5);
			double zonaWidth = recinteRenderer.getZonaWidth(this.zona, 600, 5);
			double zonaHeight = recinteRenderer.getZonaHeight(this.zona, 500, 5);
			
			double dialogTop = zonaTop + 105 + (zonaHeight - 90)/2;
			double dialogLeft = zonaLeft + 10 + (zonaWidth - 250)/2;
			
			Dialog preuDialog = (Dialog) recinteRenderer.findComponent("zones_zona_dialog");
			preuDialog.setPosition(dialogLeft+","+dialogTop);
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		return null;
	}
	
	public String toggleLocalitat(Localitat loc) {
		try {
			// Si la localitat està ocupada o bloquejada no fem res
			if (!isLocalitatOcupada(loc) && !isLocalitatBloquejada(loc)) {
				// Si la localitat és d'una zona sense preu assignat, avisem
				if (getPreuLocalitat(loc) == "No definit") {
					FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "No es poden reservar localitats de zones sense un preu per localitat definit." + messageCloseButton, "");
		            FacesContext.getCurrentInstance().addMessage(null, message);
		            return null;
				}
				// Si la localitat és d'una zona diferent a la de les altres localitats seleccionades...
				if (isLocalitatZonaDiferent(loc)) {
					// Si l'altra zona seleccionada és numerada, avisem que no es pot seleccionar
					if (this.zona == null || (this.zona.getNumFiles() != null && this.zona.getNumSeientsFila() != null)) {
						FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "No es poden reservar localitats de diferents zones." + messageCloseButton, "");
			            FacesContext.getCurrentInstance().addMessage(null, message);
			            return null;
					}
				}
				// Si la localitat està seleccionada, la deseleccionem
				if (isLocalitatSeleccionada(loc)) {
					for (Localitat localitat : this.reserva.getLocalitats()) {
						if (localitat.getZona().getId() == loc.getZona().getId() && localitat.getFila() == loc.getFila() && localitat.getSeient() == loc.getSeient()) {
							Zona zona = this.reserva.getZona();
							zona.getLocalitats().remove(localitat);
							zonaDAO.unir(zona);
							
							List<Localitat> locsReserva = this.reserva.getLocalitats();
							locsReserva.remove(localitat);
							
							List<Localitat> locsReservaNet = new ArrayList<Localitat>();
							locsReservaNet.addAll(locsReserva);
							
							this.reserva.setLocalitats(locsReservaNet);
							this.reserva.setNumLocalitats(this.reserva.getLocalitats().size());
							this.reserva.setEstat(EstatReserva.TEMPORAL);
							this.reserva.setDataUltModificacio(new Date());
							
							this.reserva = reservaDAO.unir(this.reserva);
							
							localitatDAO.eliminar(localitat);
							
							if (loginMB.getTipusUser().equalsIgnoreCase("CLIENT")) {
								Client client = clientDAO.cercarPerId(((Client) loginMB.getLoggedUser()).getId());
								loginMB.setLoggedUser(client);
							} else if (loginMB.getTipusUser().equalsIgnoreCase("TAQUILLER")) {
								Taquiller taquiller = taquillerDAO.cercarPerId(((Taquiller) loginMB.getLoggedUser()).getId());
								loginMB.setLoggedUser(taquiller);
							}
							
							break;
						}
					}
				// Si la localitat no està seleccionada, la seleccionem
				} else {
					// Editem la reserva si ja existeix i la possem en estat TEMPORAL
					if (this.reserva.getId() != null) {
						loc.setReserva(this.reserva);
						
						this.zona = loc.getZona();
						
						this.reserva.getLocalitats().add(loc);
						this.reserva.setZona(this.zona);
						this.reserva.setNumLocalitats(this.reserva.getLocalitats().size());
						this.reserva.setEstat(EstatReserva.TEMPORAL);
						this.reserva.setDataUltModificacio(new Date());
						
						this.reserva = reservaDAO.unir(this.reserva);
						
						if (loginMB.getTipusUser().equalsIgnoreCase("CLIENT")) {
							Client client = clientDAO.cercarPerId(((Client) loginMB.getLoggedUser()).getId());
							loginMB.setLoggedUser(client);
						} else if (loginMB.getTipusUser().equalsIgnoreCase("TAQUILLER")) {
						}
					// Cal crear una nova reserva TEMPORAL
					} else {
						Client client = new Client();
						if (loginMB.getTipusUser().equalsIgnoreCase("CLIENT")) {
							client = (Client) loginMB.getLoggedUser();
						} else if (loginMB.getTipusUser().equalsIgnoreCase("TAQUILLER")) {
							client = this.clientReserva;
						}
						
						List<Localitat> localitats = new ArrayList<Localitat>();
						localitats.add(loc);
						
						this.zona = loc.getZona();
						
						this.reserva.setFuncio(this.funcio);
						this.reserva.setZona(this.zona);
						this.reserva.setClient(client);
						this.reserva.setCodiQR("");
						this.reserva.setPreuFinal(0);
						this.reserva.setEstat(EstatReserva.TEMPORAL);
						this.reserva.setLocalitats(localitats);
						this.reserva.setNumLocalitats(1);
						this.reserva.setDataCreacio(new Date());
						
						loc.setReserva(this.reserva);
						
						this.reserva = reservaDAO.persistir(this.reserva);
						
						client.getReserves().add(this.reserva);
						if (loginMB.getTipusUser().equalsIgnoreCase("CLIENT")) {
							client = clientDAO.cercarPerId(client.getId());
							loginMB.setLoggedUser(client);
						} else if (loginMB.getTipusUser().equalsIgnoreCase("TAQUILLER")) {
							Taquiller taquiller = taquillerDAO.cercarPerId(((Taquiller) loginMB.getLoggedUser()).getId());
							loginMB.setLoggedUser(taquiller);
						}
						
						this.idReservaModificar = this.reserva.getId();
					}
				}
				
				String url = "/views/app/reserva/localitats.xhtml?faces-redirect=true"+
						"&novaReserva="+this.novaReserva+
						"&desdeInici="+this.desdeInici+
						"&idReservaModificar="+this.reserva.getId()+
						"&idEspectacle="+this.espectacle.getId()+
						"&idFuncio="+this.funcio.getId()+
						"&idZona="+this.zona.getId();
				
				if (this.idClientReserva != null) url += "&idClientReserva="+this.idClientReserva;
				
				return url;
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		return null;
	}
	
	public boolean isLocalitatZonaDiferent(Localitat loc) {
		if (this.zona != null) {
			return loc.getZona().getId() != this.zona.getId();
		}
		
		return false;
	}
	
	public boolean isLocalitatSeleccionada(Localitat loc) {
		for (Localitat localitat : this.localitatsSeleccionades) {
			if (localitat.getZona().getId() == loc.getZona().getId() && localitat.getFila() == loc.getFila() && localitat.getSeient() == loc.getSeient()) {
				return true;
			}
		}
		
		return false;
	}
	
	public boolean isLocalitatOcupada(Localitat loc) {
		for (Localitat localitat : this.localitatsOcupades) {
			if (localitat.getZona().getId() == loc.getZona().getId() && localitat.getFila() == loc.getFila() && localitat.getSeient() == loc.getSeient()) {
				return true;
			}
		}
		
		return false;
	}
	
	public boolean isLocalitatBloquejada(Localitat loc) {
		for (Localitat localitat : this.localitatsBloquejades) {
			if (localitat.getZona().getId() == loc.getZona().getId() && localitat.getFila() == loc.getFila() && localitat.getSeient() == loc.getSeient()) {
				return true;
			}
		}
		
		return false;
	}
	
	public String getMessageCloseButton() {
		return messageCloseButton;
	}
	
	public LoginMB getLoginMB() {
		return loginMB;
	}
	
	public void setLoginMB(LoginMB loginMB) {
		this.loginMB = loginMB;
	}
	
	public Reserva getReserva() {
		return reserva;
	}
	
	public void setReserva(Reserva reserva) {
		this.reserva = reserva;
	}
	
	public Espectacle getEspectacle() {
		return espectacle;
	}
	
	public void setEspectacle(Espectacle espectacle) {
		this.espectacle = espectacle;
	}
	
	public Funcio getFuncio() {
		return funcio;
	}
	
	public void setFuncio(Funcio funcio) {
		this.funcio = funcio;
	}
	
	public Zona getZona() {
		return zona;
	}
	
	public void setZona(Zona zona) {
		this.zona = zona;
	}
	
	public Client getClientReserva() {
		return clientReserva;
	}
	
	public void setClientReserva(Client clientReserva) {
		this.clientReserva = clientReserva;
	}
	
	public Descompte getDescompte() {
		return descompte;
	}
	
	public void setDescompte(Descompte descompte) {
		this.descompte = descompte;
	}
	
	public Valoracio getValoracio() {
		return valoracio;
	}
	
	public void setValoracio(Valoracio valoracio) {
		this.valoracio = valoracio;
	}
	
	public String getNomEspectacleCerca() {
		return nomEspectacleCerca;
	}
	
	public void setNomEspectacleCerca(String nomEspectacleCerca) {
		this.nomEspectacleCerca = nomEspectacleCerca;
	}
	
	public Integer getIdReservaModificar() {
		return idReservaModificar;
	}
	
	public void setIdReservaModificar(Integer idReservaModificar) {
		this.idReservaModificar = idReservaModificar;
	}
	
	public Integer getIdEspectacle() {
		return idEspectacle;
	}
	
	public void setIdEspectacle(Integer idEspectacle) {
		this.idEspectacle = idEspectacle;
	}
	
	public Integer getIdFuncio() {
		return idFuncio;
	}
	
	public void setIdFuncio(Integer idFuncio) {
		this.idFuncio = idFuncio;
	}
	
	public Integer getIdZona() {
		return idFuncio;
	}
	
	public void setIdZona(Integer idZona) {
		this.idZona = idZona;
	}
	
	public Integer getIdClientReserva() {
		return idClientReserva;
	}
	
	public void setIdClientReserva(Integer idClientReserva) {
		this.idClientReserva = idClientReserva;
	}
	
	public boolean getEditarNumLocsZonaNoNumerada() {
		return editarNumLocsZonaNoNumerada;
	}
	
	public void setEditarNumLocsZonaNoNumerada(boolean editarNumLocsZonaNoNumerada) {
		this.editarNumLocsZonaNoNumerada = editarNumLocsZonaNoNumerada;
	}
	
	public boolean getNovaReserva() {
		return novaReserva;
	}
	
	public void setNovaReserva(boolean novaReserva) {
		this.novaReserva = novaReserva;
	}
	
	public boolean getDesdeInici() {
		return desdeInici;
	}
	
	public void setDesdeInici(boolean desdeInici) {
		this.desdeInici = desdeInici;
	}
	
	public List<Reserva> getReserves() {
		return reserves;
	}
	
	public void setReserves(List<Reserva> reserves) {
		this.reserves = reserves;
	}
	
	public List<Espectacle> getEspectacles() {
		return espectacles;
	}
	
	public void setEspectacles(List<Espectacle> espectacles) {
		this.espectacles = espectacles;
	}
	
	public List<SelectItem> getOpcionsTipusEspectacleLlistat() {
		return opcionsTipusEspectacleLlistat;
	}
	
	public void setOpcionsTipusEspectacleLlistat(List<SelectItem> opcionsTipusEspectacleLlistat) {
		this.opcionsTipusEspectacleLlistat = opcionsTipusEspectacleLlistat;
	}
	
	public List<SelectItem> getOpcionsPuntuacioValoracio() {
		return opcionsPuntuacioValoracio;
	}
	
	public void setOpcionsPuntuacioValoracio(List<SelectItem> opcionsPuntuacioValoracio) {
		this.opcionsPuntuacioValoracio = opcionsPuntuacioValoracio;
	}
	
	public List<Zona> getZones() {
		return zones;
	}
	
	public void setZones(List<Zona> zones) {
		this.zones = zones;
	}

}