package com.espectaclia.managed_beans;

import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.faces.application.ConfigurableNavigationHandler;
import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.ViewScoped;
import javax.faces.component.UIInput;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.servlet.ServletContext;

import com.espectaclia.entities.Administrador;
import com.espectaclia.entities.Recinte;
import com.espectaclia.entities.Taquiller;
import com.espectaclia.entities.Usuari;
import com.espectaclia.daos.RecinteDAO;
import com.espectaclia.daos.TaquillerDAO;
import com.espectaclia.daos.UsuariDAO;

import org.primefaces.event.SelectEvent;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.context.support.WebApplicationContextUtils;

@ManagedBean(name="taquillerMB")
@ViewScoped
public class TaquillerMB implements Serializable{

	private static final long serialVersionUID = 1L;
	private static final String messageCloseButton = "<button type=\"button\" class=\"close\" onclick=\"$(this).parent().fadeOut()\"><span aria-hidden=\"true\">×</span></button>";
	
	@ManagedProperty(value="#{loginMB}")
    private LoginMB loginMB;
	
	@Autowired
	private UsuariDAO usuariDAO;
	
	@Autowired
	private TaquillerDAO taquillerDAO;

	@Autowired
	private RecinteDAO recinteDAO;
	
	private Taquiller taquiller;
	private Taquiller taquillerSel;
	private Integer idTaquillerModificar;
	
	private List<Taquiller> taquillers;
	private List<Recinte> recintes;
	
	@PostConstruct
	public void init() {
		// Importem l'Autowired (Spring) al ManagedBean (JSF) per poder injectar els DAOs
		ExternalContext externalContext = FacesContext.getCurrentInstance().getExternalContext();
        ServletContext servletContext = (ServletContext) externalContext.getContext();
        WebApplicationContextUtils.getRequiredWebApplicationContext(servletContext).getAutowireCapableBeanFactory().autowireBean(this);
		
        taquiller = new Taquiller();
		taquillerSel = null;
		idTaquillerModificar = null;
		taquillers = new ArrayList<Taquiller>();
    }
	
	public void loadTaquillers() {
		this.taquiller = new Taquiller();
		this.taquillers = new ArrayList<Taquiller>();

		try {
			this.taquillers.addAll(taquillerDAO.cercarPerPropietat("administrador", loginMB.getLoggedUser()));
			
			// Si es tracta d'una modificació, busquem el taquiller a modificar
			if (this.idTaquillerModificar != null) {
				for (Taquiller taq : this.taquillers) {
					if (taq.getId() == this.idTaquillerModificar) {
						this.taquiller = taq;
						break;
					}
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public void loadRecintes() {
		this.recintes = new ArrayList<Recinte>();
		
		try {
			this.recintes.addAll(recinteDAO.cercarPerPropietat("administrador", loginMB.getLoggedUser()));
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public String crear() {
		try {
			// Comprovem si ja existeix un altre usuari amb el username indicat
			List<Usuari> usuaris = usuariDAO.cercarPerPropietat("username", taquiller.getUsername());
			if (usuaris.size() > 0) {
				UIInput uiInputUsuari = (UIInput) FacesContext.getCurrentInstance().getViewRoot().findComponent("taquiller_form:taquiller_usuari_input");
				uiInputUsuari.setValid(false);
				
				FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "L'usuari \""+taquiller.getUsername()+"\" ja existeix. Tria un altre usuari." + messageCloseButton, "");
	            FacesContext.getCurrentInstance().addMessage(null, message);
	            return null;
			}
			
			taquiller.setAdministrador((Administrador) loginMB.getLoggedUser());
			taquiller.setDataAlta(new Date());
			
			taquillerDAO.persistir(this.taquiller);

			// El nou taquiller s'ha creat amb èxit
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_INFO, "El taquiller s'ha creat amb èxit." + messageCloseButton, "");
			FacesContext.getCurrentInstance().addMessage(null, message);
			FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);
            return "/views/app/taquiller/taquillers.xhtml?faces-redirect=true";
		} catch (Exception e) {
			// S'ha produït un error desconegut intentant crear el nou taquiller
			e.printStackTrace();
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "No s'ha pogut crear el taquiller. Si us plau, comprova que les dades introduïdes siguin correctes." + messageCloseButton, "");
            FacesContext.getCurrentInstance().addMessage(null, message);
            return null;
		}
	}
	
	public String guardar() {
		try {
			// Comprovem si ja existeix un altre usuari amb el username indicat (que no sigui el propi taquiller)
			List<Usuari> usuaris = usuariDAO.cercarPerPropietat("username", taquiller.getUsername());
			if (usuaris.size() > 0 && usuaris.get(0).getId() != taquiller.getId()) {
				UIInput uiInputUsuari = (UIInput) FacesContext.getCurrentInstance().getViewRoot().findComponent("taquiller_form:taquiller_usuari_input");
				uiInputUsuari.setValid(false);
				
				FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "L'usuari \""+taquiller.getUsername()+"\" ja existeix. Tria un altre usuari." + messageCloseButton, "");
	            FacesContext.getCurrentInstance().addMessage(null, message);
	            return null;
			}
			
			taquiller.setDataUltModificacio(new Date());
			
			taquillerDAO.unir(this.taquiller);

			// El taquiller s'ha modificat amb èxit
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_INFO, "El taquiller s'ha modificat amb èxit." + messageCloseButton, "");
			FacesContext.getCurrentInstance().addMessage(null, message);
			FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);
			return "/views/app/taquiller/taquillers.xhtml?faces-redirect=true";
		} catch (Exception e) {
			// S'ha produït un error desconegut intentant modificar el taquiller
			e.printStackTrace();
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "No s'ha pogut modificar el taquiller. Si us plau, comprova que les dades modificades siguin correctes." + messageCloseButton, "");
            FacesContext.getCurrentInstance().addMessage(null, message);
            return null;
		}
	}
	
	public String eliminar(Taquiller taq) {
		try {
			Recinte recinte = taq.getRecinte();
			
			// Mirem si el taquiller té recinte associat i si es així l'eliminem de l'altre extrem de la relació
			if (recinte != null) {
				for (Taquiller taquillerObj : recinte.getTaquillers()) {
					if (taquillerObj.getId() == taq.getId()) {
						recinte.getTaquillers().remove(taquillerObj);
						recinteDAO.unir(recinte);
						break;
					}
				}
			}
			
			taquillerDAO.eliminar(taq.getId());
			
			// El taquiller s'ha eliminat amb èxit
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_INFO, "El taquiller s'ha eliminat amb èxit." + messageCloseButton, "");
			FacesContext.getCurrentInstance().addMessage(null, message);
			FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);
			return "/views/app/taquiller/taquillers.xhtml?faces-redirect=true";
		} catch (Exception e) {
			// S'ha produït un error desconegut intentant eliminar el taquiller
			e.printStackTrace();
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "S'ha produït un error i no s'ha pogut eliminar el taquiller." + messageCloseButton, "");
            FacesContext.getCurrentInstance().addMessage(null, message);
		}
		return null;
	}
	
	public void seleccionarTaquiller(SelectEvent event) throws IOException {
		taquillerSel = (Taquiller) event.getObject();
		
		ConfigurableNavigationHandler configurableNavigationHandler = (ConfigurableNavigationHandler)FacesContext.getCurrentInstance().getApplication().getNavigationHandler();
		configurableNavigationHandler.performNavigation("/views/app/inici.xhtml?faces-redirect=true&idTaquiller="+taquillerSel.getId());
	}
	
	public String nouTaquiller() {
		return "/views/app/taquiller/modificar_taquiller.xhtml?faces-redirect=true";
	}
	
	public String modificarTaquiller(Integer id) {
		this.idTaquillerModificar = id;
		
		return "/views/app/taquiller/modificar_taquiller.xhtml?faces-redirect=true&includeViewParams=true";
	}
	
	public List<Object> getBreadcrumbParams() {
		List<Object> params = new ArrayList<Object>();
		params.add(idTaquillerModificar);
		
		return params;
	}
	
	public String getMessageCloseButton() {
		return messageCloseButton;
	}
	
	public LoginMB getLoginMB() {
		return loginMB;
	}
	
	public void setLoginMB(LoginMB loginMB) {
		this.loginMB = loginMB;
	}
	
	public Taquiller getTaquiller() {
		return taquiller;
	}
	
	public void setTaquiller(Taquiller taquiller) {
		this.taquiller = taquiller;
	}
	
	public Taquiller getTaquillerSel() {
		return taquillerSel;
	}
	
	public void setTaquillerSel(Taquiller taquillerSel) {
		this.taquillerSel = taquillerSel;
	}
	
	public Integer getIdTaquillerModificar() {
		return idTaquillerModificar;
	}
	
	public void setIdTaquillerModificar(Integer idTaquillerModificar) {
		this.idTaquillerModificar = idTaquillerModificar;
	}
	
	public List<Taquiller> getTaquillers() {
		return taquillers;
	}
	
	public void setTaquillers(List<Taquiller> taquillers) {
		this.taquillers = taquillers;
	}
	
	public List<Recinte> getRecintes() {
		return recintes;
	}
	
	public void setRecintes(List<Recinte> recintes) {
		this.recintes = recintes;
	}

}