package com.espectaclia.managed_beans;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ViewScoped;
import javax.faces.component.UIComponent;
import javax.faces.component.UIInput;
import javax.faces.component.UIOutput;
import javax.faces.component.html.HtmlInputText;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.servlet.ServletContext;

import com.espectaclia.entities.Localitat;
import com.espectaclia.entities.Zona;
import com.espectaclia.entities.Recinte;
import com.espectaclia.utils.RecinteRenderer;
import com.espectaclia.daos.ZonaDAO;
import com.espectaclia.daos.RecinteDAO;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.context.support.WebApplicationContextUtils;

@ManagedBean(name="zonaMB")
@ViewScoped
public class ZonaMB implements Serializable{

	private static final long serialVersionUID = 1L;
	private static final String messageCloseButton = "<button type=\"button\" class=\"close\" onclick=\"$(this).parent().fadeOut()\"><span aria-hidden=\"true\">×</span></button>";
	
	@Autowired
	private ZonaDAO zonaDAO;
	
	@Autowired
	private RecinteDAO recinteDAO;
	
	@Autowired
	private RecinteRenderer recinteRenderer;

	private Recinte recinte;
	private Zona zona;
	private Integer idRecinte;
	private Integer idZonaModificar;
	
	private boolean localitatsNumerades;
	
	private List<Zona> zones;
	
	@PostConstruct
	public void init() {
		// Importem l'Autowired (Spring) al ManagedBean (JSF) per poder injectar els DAOs
		ExternalContext externalContext = FacesContext.getCurrentInstance().getExternalContext();
        ServletContext servletContext = (ServletContext) externalContext.getContext();
        WebApplicationContextUtils.getRequiredWebApplicationContext(servletContext).getAutowireCapableBeanFactory().autowireBean(this);
		
        recinte = new Recinte();
		zona = new Zona();
		zones = new ArrayList<Zona>();
    }
	
	public void loadRecinte() {
		try {
			if (this.idRecinte != null) {
				this.recinte = recinteDAO.cercarPerId(this.idRecinte);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public void loadZones() {
		this.zona = new Zona();
		this.zones = new ArrayList<Zona>();
		
		try {
			this.zones.addAll(zonaDAO.cercarPerPropietat("recinte", this.recinte));
			
			// Generem les localitats per pintar el pati de butaques de les zones amb localitats numerades
			for (Zona zon : this.zones) {
				if (zon.getNumFiles() != null && zon.getNumSeientsFila() != null) {
					List<Localitat> localitats = new ArrayList<Localitat>();
					
					for (int numFila=1; numFila<=zon.getNumFiles(); numFila++) {
						for (int numSeient=1; numSeient<=zon.getNumSeientsFila(); numSeient++) {
							Localitat loc = new Localitat();
							loc.setFila(numFila);
							loc.setSeient(numSeient);
							
							localitats.add(loc);
						}
					}
					
					zon.setLocalitatsPatiButaques(localitats);
				}
			}
			
			// Si es tracta d'una modificació, busquem la zona a modificar
			if (this.idZonaModificar != null) {
				for (Zona zon : this.zones) {
					if (zon.getId() == this.idZonaModificar) {
						this.zona = zon;
						this.zones.remove(zon);
						
						this.localitatsNumerades = (this.zona.getNumFiles() != null && this.zona.getNumSeientsFila() != null ? true : false);
						
						break;
					}
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public String crear() {
		try {
			// Comprovem si ja existeix un altre zona amb el mateix nom del mateix recinte
			if (this.zones.size() > 0) {
				for (Zona zonaObj : this.zones) {
					if (zonaObj.getNom().equalsIgnoreCase(zona.getNom())) {
						UIInput uiInputUsuari = (UIInput) FacesContext.getCurrentInstance().getViewRoot().findComponent("zona_form:zona_nom_input");
						uiInputUsuari.setValid(false);
						
						FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "El recinte "+recinte.getNom()+" ja té una zona amb el nom \""+zona.getNom()+"\". Tria un altre nom." + messageCloseButton, "");
			            FacesContext.getCurrentInstance().addMessage(null, message);
			            return null;
					}
				}
			}
			
			if (this.zona.getCoordInici() != null) this.zona.setCoordInici(this.zona.getCoordInici().toUpperCase());
			if (this.zona.getCoordFi() != null) this.zona.setCoordFi(this.zona.getCoordFi().toUpperCase());
			this.zona.setRecinte(recinte);
			
			zonaDAO.persistir(this.zona);

			// El nou zona s'ha creat amb èxit
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_INFO, "La zona s'ha creat amb èxit." + messageCloseButton, "");
			FacesContext.getCurrentInstance().addMessage(null, message);
			FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);
            return tornarGestioZones();
		} catch (Exception e) {
			// S'ha produït un error desconegut intentant crear el nou zona
			e.printStackTrace();
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "No s'ha pogut crear la zona. Si us plau, comprova que les dades introduïdes siguin correctes." + messageCloseButton, "");
            FacesContext.getCurrentInstance().addMessage(null, message);
            return null;
		}
	}
	
	public String guardar() {
		try {
			if (this.zona.getCoordInici() != null) this.zona.setCoordInici(this.zona.getCoordInici().toUpperCase());
			if (this.zona.getCoordFi() != null) this.zona.setCoordFi(this.zona.getCoordFi().toUpperCase());
			
			// Comprovem si ja existeix una altra zona amb el mateix nom del mateix recinte
			if (this.zones.size() > 0) {
				for (Zona zonaObj : this.zones) {
					if (zonaObj.getNom().equalsIgnoreCase(zona.getNom()) && zonaObj.getId() != zona.getId()) {
						UIInput uiInputUsuari = (UIInput) FacesContext.getCurrentInstance().getViewRoot().findComponent("zona_form:zona_nom_input");
						uiInputUsuari.setValid(false);
						
						FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "El recinte "+recinte.getNom()+" ja té una zona amb el nom \""+zona.getNom()+"\". Tria un altre nom." + messageCloseButton, "");
			            FacesContext.getCurrentInstance().addMessage(null, message);
			            return null;
					}
				}
			}
			
			zonaDAO.unir(this.zona);

			// La zona s'ha modificat amb èxit
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_INFO, "La zona s'ha modificat amb èxit." + messageCloseButton, "");
			FacesContext.getCurrentInstance().addMessage(null, message);
			FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);
			return tornarGestioZones();
		} catch (Exception e) {
			// S'ha produït un error desconegut intentant modificar la zona
			e.printStackTrace();
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "No s'ha pogut modificar la zona. Si us plau, comprova que les dades modificades siguin correctes." + messageCloseButton, "");
            FacesContext.getCurrentInstance().addMessage(null, message);
            return null;
		}
	}
	
	public String eliminar(Zona zon) {
		try {
			// Comprovem que la zona no tingui cap reserva pendent
			if (zon.teReservesPendents(null)) {
				FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "La zona no es pot eliminar perquè té reserves pendents.", "");
	            FacesContext.getCurrentInstance().addMessage(null, message);
	            return null;
			}
			
			// Eliminem la zona de la llista d'zones del recinte
			if (recinte != null) {
				for (Zona zona : this.recinte.getZones()) {
					if (zona.getId() == zon.getId()) {
						recinte.getZones().remove(zona);
						recinteDAO.unir(recinte);
						break;
					}
				}
			}
			
			// La zona s'ha eliminat amb èxit
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_INFO, "La zona s'ha eliminat amb èxit." + messageCloseButton, "");
			FacesContext.getCurrentInstance().addMessage(null, message);
			FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);
			return tornarGestioZones();
		} catch (Exception e) {
			// S'ha produït un error desconegut intentant eliminar la zona
			e.printStackTrace();
			FacesMessage message = new FacesMessage(FacesMessage.SEVERITY_ERROR, "S'ha produït un error i no s'ha pogut eliminar la zona." + messageCloseButton, "");
            FacesContext.getCurrentInstance().addMessage(null, message);
		}
		return null;
	}
	
	public String tornarGestioZones() {
		return "/views/app/zona/zones.xhtml?faces-redirect=true&idRecinte="+idRecinte;
	}
	
	public String novaZona() {
		return "/views/app/zona/modificar_zona.xhtml?faces-redirect=true&idRecinte="+idRecinte;
	}
	
	public String modificarZona(Integer id) {
		return "/views/app/zona/modificar_zona.xhtml?faces-redirect=true&idRecinte="+idRecinte+"&idZonaModificar="+id;
	}
	
	public List<Object> getBreadcrumbParams() {
		List<Object> params = new ArrayList<Object>();
		params.add(idRecinte);
		params.add(idZonaModificar);
		
		return params;
	}
	
	public void canviLocalitatsNumerades() {
		HtmlInputText numFiles = (HtmlInputText) recinteRenderer.findComponent("zona_files_input");
		numFiles.setDisabled(!this.localitatsNumerades);
		numFiles.setRequired(this.localitatsNumerades);
		numFiles.setValue(null);
		
		HtmlInputText numSeientsFila = (HtmlInputText) recinteRenderer.findComponent("zona_seients_fila_input");
		numSeientsFila.setDisabled(!this.localitatsNumerades);
		numSeientsFila.setRequired(this.localitatsNumerades);
		numSeientsFila.setValue(null);
		
		if (!this.localitatsNumerades) {
			this.zona.setNumFiles(null);
			this.zona.setNumSeientsFila(null);
		}
	}
	
	public void canviZonaPreview() {
		try {
			if (this.zona.getCoordInici() != null) this.zona.setCoordInici(this.zona.getCoordInici().toUpperCase());
			if (this.zona.getCoordFi() != null) this.zona.setCoordFi(this.zona.getCoordFi().toUpperCase());
			
			UIComponent zonaModificar = recinteRenderer.findComponent("zona_modificar");
			zonaModificar.getAttributes().put("style", getZonaStyle(this.zona, 330, 290, 2));
			
			if (this.zona.getNom() != null) {
				UIOutput nom = (UIOutput) zonaModificar.findComponent("zona_modificar_nom");
				nom.setValue(this.zona.getNom());
			}
			
			if (this.zona.getCoordInici() != null && this.zona.getCoordFi() != null) {
				UIOutput coords = (UIOutput) zonaModificar.findComponent("zona_modificar_coords");
				coords.setValue("("+this.zona.getCoordInici()+" x "+this.zona.getCoordFi()+")");
			}
		} catch (Exception e) {}
	}
	
	public String getZonaStyle(Zona zona, int ampladaRecinte, int alturaRecinte, int margeLateral) {
		if (zona.getCoordInici() != null && zona.getCoordInici().trim().length() > 0 && zona.getCoordFi() != null && zona.getCoordFi().trim().length() > 0) {
			double left = recinteRenderer.getZonaLeft(zona, ampladaRecinte, margeLateral);
			double top = recinteRenderer.getZonaTop(zona, alturaRecinte, margeLateral);
			double width = recinteRenderer.getZonaWidth(zona, ampladaRecinte, margeLateral);
			double height = recinteRenderer.getZonaHeight(zona, alturaRecinte, margeLateral);
			
			return "left: "+left+"px;"+
				"top: "+top+"px;"+
				"width: "+width+"px;"+
				"height: "+height+"px;";
		} else {
			return "display: none";
		}
	}
	
	public String getLocalitatZonaStyle(Localitat localitat, Zona zona, int ampladaRecinte, int alturaRecinte, int margeLateralZona, int margeLateralLocalitat) {
		double left = recinteRenderer.getLocalitatZonaLeft(localitat, zona, ampladaRecinte, margeLateralZona, margeLateralLocalitat);
		double top = recinteRenderer.getLocalitatZonaTop(localitat, zona, alturaRecinte, margeLateralZona, margeLateralLocalitat);
		double width = recinteRenderer.getLocalitatZonaWidth(localitat, zona, ampladaRecinte, margeLateralZona, margeLateralLocalitat);
		double height = recinteRenderer.getLocalitatZonaHeight(localitat, zona, alturaRecinte, margeLateralZona, margeLateralLocalitat);
		
		return "left: "+left+"px;"+
				"top: "+top+"px;"+
				"width: "+width+"px;"+
				"height: "+height+"px;";
	}
	
	public String getMessageCloseButton() {
		return messageCloseButton;
	}
	
	public Recinte getRecinte() {
		return recinte;
	}
	
	public void setRecinte(Recinte recinte) {
		this.recinte = recinte;
	}
	
	public Zona getZona() {
		return zona;
	}
	
	public void setZona(Zona zona) {
		this.zona = zona;
	}
	
	public Integer getIdRecinte() {
		return idRecinte;
	}
	
	public void setIdRecinte(Integer idRecinte) {
		this.idRecinte = idRecinte;
	}
	
	public Integer getIdZonaModificar() {
		return idZonaModificar;
	}
	
	public void setIdZonaModificar(Integer idZonaModificar) {
		this.idZonaModificar = idZonaModificar;
	}
	
	public boolean getLocalitatsNumerades() {
		return localitatsNumerades;
	}
	
	public void setLocalitatsNumerades(boolean localitatsNumerades) {
		this.localitatsNumerades = localitatsNumerades;
	}
	
	public List<Zona> getZones() {
		return zones;
	}
	
	public void setZones(List<Zona> zones) {
		this.zones = zones;
	}

}