package com.espectaclia.security;

import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.espectaclia.daos.UsuariDAO;
import com.espectaclia.entities.Usuari;

@Service
@Transactional(readOnly=true)
public class CustomUserDetailsService implements UserDetailsService {

	@Autowired
    private UsuariDAO usuariDAO;
	
	public UserDetails loadUserByUsername(String username)
			throws UsernameNotFoundException, DataAccessException {

		Usuari usuari = null;
		String tipus = null;
		
		try {
			// No es pot fer login amb l'usuari especial 'BLOQUEIG' (s'utilitza com a dummy per a bloquejar localitats de reserves)
			if (username.equalsIgnoreCase("BLOQUEIG")) throw new UsernameNotFoundException("Error intentant autentificar l'usuari.");
			
			List<Usuari> usuaris = usuariDAO.cercarPerPropietat("username", username);
			
			if (usuaris.size() > 0) {
				usuari = usuaris.get(0);
				
				String className = usuari.getClass().getSimpleName();
				if (className.compareTo("Client") == 0) {
					tipus = "CLIENT";
				} else if (className.compareTo("Taquiller") == 0) {
					tipus = "TAQUILLER";
				} else if (className.compareTo("Administrador") == 0) {
					tipus = "ADMINISTRADOR";
				}
			}
		} catch (Exception e) {
			throw new UsernameNotFoundException("Error intentant autentificar l'usuari.");
		}
		
		if (usuari != null) {
			boolean enabled = true;
	        boolean accountNonExpired = true;
	        boolean credentialsNonExpired = true;
	        boolean accountNonLocked = true;
	        
	        List<GrantedAuthority> authList = new ArrayList<GrantedAuthority>();
	        authList.add(new SimpleGrantedAuthority(tipus));
	        
	        return new CustomUserDetails(
	        	usuari,
	        	tipus,
	        	enabled,
	        	accountNonExpired,
	        	credentialsNonExpired,
	        	accountNonLocked,
	        	authList
	        );
		} else {
			throw new UsernameNotFoundException("L'usuari no existeix.");
		}
	}
}