package com.espectaclia.services;

import java.util.List;

import com.espectaclia.entities.Client;
import com.espectaclia.entities.Reserva;
import com.espectaclia.entities.Taquiller;
import com.espectaclia.daos.ReservaDAO;

import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

@Component
@Transactional
public class ReservaService extends EntityService<Reserva> implements ReservaDAO {

	@Transactional
	public void eliminar(Reserva reserva) throws Exception{
		if (!getEntityManager().contains(reserva)){
		   reserva = getEntityManager().merge(reserva);
		}
		
		if (reserva.getLocalitats().size() > 0) {
			reserva.getZona().getLocalitats().removeAll(reserva.getLocalitats());
			reserva.getLocalitats().clear();
		}

		reserva.getZona().getReserves().remove(reserva);
		reserva.getFuncio().getReserves().remove(reserva);
		reserva.getClient().getReserves().remove(reserva);

		getEntityManager().remove(reserva);
	};
	
	@Transactional
	@SuppressWarnings("unchecked")
	public List<Reserva> cercarReservesClient(Client client) throws Exception{
		// Estats: 0 - BLOQUEJADA, 1 - TEMPORAL, 2 - VENUDA, 3 - CANCELLADA, 4 - CONFIRMADA
		return (List<Reserva>) getEntityManager().createQuery("select r from " + getEntityClass().getSimpleName() + " r where r.client = ?1 and r.estat in (2,3,4)").setParameter(1, client).getResultList();
	};
	
	@Transactional
	@SuppressWarnings("unchecked")
	public List<Reserva> cercarReservesTaquiller(Taquiller taquiller) throws Exception{
		// Estats: 0 - BLOQUEJADA, 1 - TEMPORAL, 2 - VENUDA, 3 - CANCELLADA, 4 - CONFIRMADA
		return (List<Reserva>) getEntityManager().createQuery("select r from " + getEntityClass().getSimpleName() + " r where r.zona.recinte = ?1 and r.estat in (2,3,4)").setParameter(1, taquiller.getRecinte()).getResultList();
	};
	
}