package com.espectaclia.utils;

import javax.faces.component.UIComponent;
import javax.faces.component.UIViewRoot;
import javax.faces.component.visit.VisitCallback;
import javax.faces.component.visit.VisitContext;
import javax.faces.component.visit.VisitResult;
import javax.faces.context.FacesContext;

import org.springframework.stereotype.Component;

import com.espectaclia.entities.Localitat;
import com.espectaclia.entities.Zona;
import com.sun.faces.component.visit.FullVisitContext;

@Component
public class RecinteRenderer {
	
	private static final int numPosicionsH = 12;
	private static final int numPosicionsV = 10;
	
	public double getZonaLeft(Zona zona, int ampladaRecinte, int margeLateral) {
		int numInici = Integer.parseInt(zona.getCoordInici().substring(1));
		
		double ampladaUnitat = ampladaRecinte / numPosicionsH;
		double left = (numInici - 1) * ampladaUnitat + margeLateral;
		
		return left;
	}
	
	public double getZonaTop(Zona zona, int alturaRecinte, int margeLateral) {
		String lletraInici = zona.getCoordInici().substring(0,1);
		
		double alturaUnitat = alturaRecinte / numPosicionsV;
		double top = ((int) lletraInici.charAt(0) - 65) * alturaUnitat + margeLateral;

		return top;
	}
	
	public double getZonaWidth(Zona zona, int ampladaRecinte, int margeLateral) {
		int numInici = Integer.parseInt(zona.getCoordInici().substring(1));
		int numFi = Integer.parseInt(zona.getCoordFi().substring(1));
		
		double ampladaUnitat = ampladaRecinte / numPosicionsH;
		double numUnitatsH = numFi - numInici + 1;
		double width = (numUnitatsH * ampladaUnitat) - (2 * margeLateral);
		
		return width;
	}
	
	public double getZonaHeight(Zona zona, int alturaRecinte, int margeLateral) {
		String lletraInici = zona.getCoordInici().substring(0,1);
		String lletraFi = zona.getCoordFi().substring(0,1);
		
		double alturaUnitat = alturaRecinte / numPosicionsV;
		double numUnitatsV = (int) lletraFi.charAt(0) - (int) lletraInici.charAt(0) + 1;
		double height = (numUnitatsV * alturaUnitat) - (2 * margeLateral);
		
		return height;
	}
	
	public double getLocalitatZonaLeft(Localitat localitat, Zona zona, int ampladaRecinte, int margeLateralZona, int margeLateralLocalitat) {
		double ampladaZona = getZonaWidth(zona, ampladaRecinte, margeLateralZona) - 14;
		double ampladaLocalitat = ampladaZona / zona.getNumSeientsFila();
		double left = (localitat.getSeient() - 1) * ampladaLocalitat + margeLateralLocalitat;
		
		return left;
	}
	
	public double getLocalitatZonaTop(Localitat localitat, Zona zona, int alturaRecinte, int margeLateralZona, int margeLateralLocalitat) {
		double alturaZona = getZonaHeight(zona, alturaRecinte, margeLateralZona) - 14 - 19;
		double alturaLocalitat = alturaZona / zona.getNumFiles();
		double top = (localitat.getFila() - 1) * alturaLocalitat + margeLateralLocalitat;
		
		return top;
	}
	
	public double getLocalitatZonaWidth(Localitat localitat, Zona zona, int ampladaRecinte, int margeLateralZona, int margeLateralLocalitat) {
		double ampladaZona = getZonaWidth(zona, ampladaRecinte, margeLateralZona) - 14;
		double ampladaLocalitat = ampladaZona / zona.getNumSeientsFila() - (2 * margeLateralLocalitat);
		
		return ampladaLocalitat;
	}
	
	public double getLocalitatZonaHeight(Localitat localitat, Zona zona, int alturaRecinte, int margeLateralZona, int margeLateralLocalitat) {
		double alturaZona = getZonaHeight(zona, alturaRecinte, margeLateralZona) - 14 - 19;
		double alturaLocalitat = alturaZona / zona.getNumFiles() - (2 * margeLateralLocalitat);
		
		return alturaLocalitat;
	}
	
	public UIComponent findComponent(final String id) {
	    FacesContext context = FacesContext.getCurrentInstance(); 
	    UIViewRoot root = context.getViewRoot();
	    final UIComponent[] found = new UIComponent[1];

	    root.visitTree(new FullVisitContext(context), new VisitCallback() {     
	        @Override
	        public VisitResult visit(VisitContext context, UIComponent component) {
	            if(component != null && component.getId() != null && component.getId().equals(id)){
	                found[0] = component;
	                return VisitResult.COMPLETE;
	            }
	            return VisitResult.ACCEPT;              
	        }
	    });

	    return found[0];
	}
	
}