package com.espectaclia.validators;

import javax.faces.application.FacesMessage;
import javax.faces.component.UIComponent;
import javax.faces.component.UIInput;
import javax.faces.context.FacesContext;
import javax.faces.validator.FacesValidator;
import javax.faces.validator.Validator;
import javax.faces.validator.ValidatorException;

@FacesValidator("contrasenyaValidator")
public class ContrasenyaValidator implements Validator {

	private static final String messageCloseButton = "<button type=\"button\" class=\"close\" onclick=\"$(this).parent().fadeOut()\"><span aria-hidden=\"true\">×</span></button>";
	
	@Override
	public void validate(FacesContext context, UIComponent component, Object value) throws ValidatorException {
	
		String contrasenya = value.toString();
		
		UIInput uiInputConfirmarContrasenya = (UIInput) component.getAttributes().get("confirmarContrasenya");
		String confirmarContrasenya = uiInputConfirmarContrasenya.getSubmittedValue().toString();
		
		// Validació contrasenya
		if (contrasenya == null || contrasenya.isEmpty()) return;
		
		// :::: Longitud contrasenya
		if (contrasenya.trim().length() < 8 || contrasenya.trim().length() > 18) {
			uiInputConfirmarContrasenya.setValid(false);
			throw new ValidatorException(new FacesMessage(FacesMessage.SEVERITY_ERROR, "La contrasenya ha de tenir entre 8 i 18 carácters." + messageCloseButton, ""));
		}
		
		// :::: Només caràcters alfanumèrics
		if (!contrasenya.trim().matches("\\p{Alnum}*")) {
		    throw new ValidatorException(new FacesMessage(FacesMessage.SEVERITY_ERROR, "La contrasenya només pot contenir carácters alfanumèrics." + messageCloseButton, ""));
		}
		
		// Validació contrasenya confirmació
		if (confirmarContrasenya == null || confirmarContrasenya.isEmpty()) return;
		
		if (!contrasenya.equals(confirmarContrasenya)) {
			uiInputConfirmarContrasenya.setValid(false);
			throw new ValidatorException(new FacesMessage(FacesMessage.SEVERITY_ERROR, "La contrasenya no coincideix amb la contrasenya de confirmació." + messageCloseButton, ""));
		}
	
	}
}
