package com.espectaclia.validators;

import javax.faces.application.FacesMessage;
import javax.faces.component.UIComponent;
import javax.faces.component.UIInput;
import javax.faces.context.FacesContext;
import javax.faces.validator.FacesValidator;
import javax.faces.validator.Validator;
import javax.faces.validator.ValidatorException;

@FacesValidator("coordsZonaValidator")
public class CoordsZonaValidator implements Validator {

	private static final String messageCloseButton = "<button type=\"button\" class=\"close\" onclick=\"$(this).parent().fadeOut()\"><span aria-hidden=\"true\">×</span></button>";
	
	@Override
	public void validate(FacesContext context, UIComponent component, Object value) throws ValidatorException {
	
		String coordFi = value.toString().toUpperCase();
		
		UIInput uiInputCoordInici = (UIInput) component.getAttributes().get("coordInici");
		String coordInici = uiInputCoordInici.getValue().toString().toUpperCase();
		
		if (coordInici == null && coordFi == null) return;
		
		String lletraInici = "";
		int numInici = 0;
		String lletraFi = "";
		int numFi = 0;
		
		// Validació coordInici
		if (coordInici != null) {
			// :::: Format coordInici (lletra + número)
			if (coordInici.trim().matches("[A-Z]{1}\\d+")) {
				lletraInici = coordInici.substring(0,1);
				numInici = Integer.parseInt(coordInici.substring(1));
				
				// :::: Límits coordInici (A - J, 1 - 12)
				if ((int) lletraInici.charAt(0) < (int) "A".charAt(0) || (int) lletraInici.charAt(0) > (int) "J".charAt(0) || numInici < 1 || numInici > 12) {
					uiInputCoordInici.setValid(false);
					throw new ValidatorException(new FacesMessage(FacesMessage.SEVERITY_ERROR, "La coordenada inicial és incorrecte, la lletra ha d'estar entre A i J i el número entre 1 i 12." + messageCloseButton, ""));
				}
			} else {
				uiInputCoordInici.setValid(false);
				throw new ValidatorException(new FacesMessage(FacesMessage.SEVERITY_ERROR, "El format de la coordenada inicial és incorrecte, ha de ser una lletra seguida d'un número." + messageCloseButton, ""));
			}
		}
		
		// Validació coordInici
		if (coordFi != null) {
			// :::: Format coordFi (lletra + número)
			if (coordFi.trim().matches("[A-Z]{1}\\d+")) {
				lletraFi = coordFi.substring(0,1);
				numFi = Integer.parseInt(coordFi.substring(1));
				
				// :::: Límits coordFi (A - J, 1 - 12)
				if ((int) lletraFi.charAt(0) < (int) "A".charAt(0) || (int) lletraFi.charAt(0) > (int) "J".charAt(0) || numFi < 1 || numFi > 12) {
					uiInputCoordInici.setValid(false);
					throw new ValidatorException(new FacesMessage(FacesMessage.SEVERITY_ERROR, "La coordenada final és incorrecte, la lletra ha d'estar entre A i J i el número entre 1 i 12." + messageCloseButton, ""));
				}
			} else {
				uiInputCoordInici.setValid(false);
				throw new ValidatorException(new FacesMessage(FacesMessage.SEVERITY_ERROR, "El format de la coordenada final és incorrecte, ha de ser una lletra seguida d'un número." + messageCloseButton, ""));
			}
		}
		
		// coordInici ha de ser inferior a coordFi
		if (coordInici.equalsIgnoreCase(coordFi)) {
			uiInputCoordInici.setValid(false);
			throw new ValidatorException(new FacesMessage(FacesMessage.SEVERITY_ERROR, "Les coordenades de posicionament no poden ser iguals." + messageCloseButton, ""));
		} else if ((int) lletraInici.charAt(0) > (int) lletraFi.charAt(0) || numInici > numFi) {
			uiInputCoordInici.setValid(false);
			throw new ValidatorException(new FacesMessage(FacesMessage.SEVERITY_ERROR, "La coordenada inicial ha de ser inferior a la coordenada final." + messageCloseButton, ""));
		}
	
	}
}